/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.activedirectory;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.ldap.InitialLdapContext;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.apache.commons.cache.Cache;
import org.apache.commons.cache.GroupMapImpl;
import org.apache.commons.cache.MemoryStash;
import org.apache.commons.cache.SimpleCache;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.boot.PropertyList;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.realms.RealmKey;
import com.sslexplorer.realms.Realm;
import com.sslexplorer.security.User;
import com.sslexplorer.security.UserDatabaseException;

public final class ActiveDirectoryUserDatabaseConfiguration {
    private static final Log logger = LogFactory.getLog(ActiveDirectoryUserDatabaseConfiguration.class);
    private static final String COMMON_NAME = "CN=";
    private static final String CN_USERS = COMMON_NAME + "Users";
    private static final String CN_BUILTIN = COMMON_NAME + "Builtin";
    private static final String USERS_CACHE_PREFIX = "USERS_";
    private static final String ROLES_CACHE_PREFIX = "ROLES_";
    private static final String LDAP_PROTOCOL = "ldap://";
    private static final String PORT_SEPARATOR = ":";
    private static final String WILDCARD_SEARCH = "*";
    /** */
    public static final String GSSAPI_AUTHENTICATION_METHOD = "GSSAPI";
    /** */
    public static final String SIMPLE_AUTHENTICATION_METHOD = "simple";

    private final Realm realm;
    private final ActiveDirectoryPropertyManager propertyManager;
    private String domain; 
    private String controllerHost; 
    private int controllerPort; 
    private Collection<String> backupControllerHosts;
    private String serviceAuthenticationType;
    private String userAuthenticationType;
    private String serviceAccountName;
    private String serviceAccountPassword;
    private String baseDn; 
    private String protocolType;
    private boolean followReferrals;
    private Cache availablePrincipalsCache; 
    private int timeToLive; 
    private final List<String> includedOuBasesList = new ArrayList<String>(); 
    private final List<String> excludedOuBasesList = new ArrayList<String>(); 
    private boolean hasFilteredOus; 
    private boolean usernamesAreCaseSensitive; 
    private final Collection<URI> activeDirectoryUrls = new ArrayList<URI>();
    private URI lastContactedActiveDirectoryUrl;
    private int pageSize;
    private PagedResultTemplate template;

    ActiveDirectoryUserDatabaseConfiguration(Realm realm) throws IllegalArgumentException, Exception {
        this.realm = realm;
        propertyManager = new ActiveDirectoryPropertyManager(realm);
        initialize();
        postIntialize();
        refresh();
    }

    private void initialize() throws Exception {
        // Get the domain and active directory root
        setDomain(Property.getProperty(getRealmKey("activeDirectory.domain")));
        setControllerHost(Property.getProperty(getRealmKey("activeDirectory.controllerHost")));
        setControllerPort(Property.getPropertyInt(getRealmKey("activeDirectory.controllerPort")));
        setBackupControllerHosts(Property.getPropertyList(getRealmKey("activeDirectory.backupControllerHosts")));
        setServiceAuthenticationType(Property.getProperty(getRealmKey("activeDirectory.serviceAuthenticationType")));
        setUserAuthenticationType(Property.getProperty(getRealmKey("activeDirectory.userAuthenticationType")));
        setServiceAccountName(Property.getProperty(getRealmKey("activeDirectory.serviceAccountUsername")));
        setServiceAccountPassword(Property.getProperty(getRealmKey("activeDirectory.serviceAccountPassword")));
        setProtocolType(Property.getProperty(getRealmKey("activeDirectory.protocol")));
        setFollowReferrals(Boolean.valueOf(System.getProperty("sslexplorer.followADReferrals", "false")));

        setCache(Property.getPropertyInt(getRealmKey("activeDirectory.cacheMaxObjects")));
        setTimeToLive(Property.getPropertyInt(getRealmKey("activeDirectory.userCacheTTL")));

        PropertyList includedOuFilterList = Property.getPropertyList(getRealmKey("activeDirectory.organizationalUnitFilter"));
        PropertyList excludedOuFilterList = Property.getPropertyList(getRealmKey("activeDirectory.excludedOrganizationalUnitFilter"));
        setValidOus(baseDn, includedOuFilterList, excludedOuFilterList);
        setIncludeStandardUsers(Property.getPropertyBoolean(getRealmKey("activeDirectory.includeStandardUsers")));
        setIncludeBuiltInGroups(Property.getPropertyBoolean(getRealmKey("activeDirectory.includeBuiltInGroups")));

        setUsernamesAreCaseSensitive(Property.getPropertyBoolean(getRealmKey("activeDirectory.usernamesAreCaseSensitive")));
        setPageSize(Property.getPropertyInt(getRealmKey("activeDirectory.pageSize")));
        setActiveDirectoryUrls();
    }

    void postIntialize() {
        if(!isServiceAuthenticationGssApi() && !serviceAccountName.toLowerCase().endsWith(baseDn)) {
            serviceAccountName = appendBaseDn(formatUsername(serviceAccountName));
        }
        includedOuBasesList.removeAll(excludedOuBasesList); // just to make sure
        template = new PagedResultTemplate(includedOuBasesList, pageSize);
    }
    
    private static String formatUsername(String username) {
    	return username.toUpperCase().startsWith(COMMON_NAME) ? username : COMMON_NAME + username;
    }
    
    public String appendBaseDn(String commonName) {
        return commonName.toLowerCase().endsWith(baseDn.toLowerCase()) ? commonName : postfixWithComma(commonName) + baseDn;
    }
    
    private static String postfixWithComma(String value) {
        return value.endsWith(",") ? value : value + ",";
    }

    void refresh() {
        propertyManager.refresh();
    }

    public String getDomain() {
        return domain;
    }
    
    private void setDomain(String domain) throws Exception {
        this.domain = domain.toUpperCase().trim();
        if (this.domain.equals("")) {
            throw new Exception("No active directory domain configured.");
        }
        setBaseDn(splitDomain(domain));
    }

    private void setControllerHost(String controllerHost) {
        if (controllerHost.equals("")) {
            throw new IllegalArgumentException("No LDAP controller host configured.");
        }
        this.controllerHost = controllerHost;
    }

    private void setControllerPort(int controllerPort) {
        this.controllerPort = controllerPort;
    }
    
    private Collection<String> getBackupControllerHosts() {
        return backupControllerHosts;
    }
    
    private void setBackupControllerHosts(Collection<String> backupControllerHosts) {
        this.backupControllerHosts = backupControllerHosts;
    }

    private String getServiceAuthenticationType() {
        return serviceAuthenticationType;
    }

    private void setServiceAuthenticationType(String serviceAuthenticationType) {
        this.serviceAuthenticationType = serviceAuthenticationType;
    }

    boolean requiresServiceAuthentication() {
        return !"none".equals(getServiceAuthenticationType());
    }

    private boolean isServiceAuthenticationGssApi() {
        return GSSAPI_AUTHENTICATION_METHOD.equals(getServiceAuthenticationType());
    }

    boolean isUserAuthenticationGssApi() {
        return GSSAPI_AUTHENTICATION_METHOD.equals(getUserAuthenticationType());
    }
    
    String getUserAuthenticationType() {
        return userAuthenticationType;
    }

    void setUserAuthenticationType(String userAuthenticationType) {
        this.userAuthenticationType = userAuthenticationType;
    }
    
    private String getServiceAccountName() {
        return serviceAccountName;
    }

    void setServiceAccountName(String serviceAccountName) {
        this.serviceAccountName = serviceAccountName == null ? null : serviceAccountName.trim();
    }

    private String getServiceAccountPassword() {
        return serviceAccountPassword;
    }

    void setServiceAccountPassword(String serviceAccountPassword) {
        this.serviceAccountPassword = serviceAccountPassword;
    }

    private String getProtocolType() {
        return protocolType;
    }

   public  boolean isSslProtcolType() {
        return "ssl".equals(getProtocolType());
    }
    
    void setProtocolType(String protocolType) {
        this.protocolType = protocolType;
    }
    
    private boolean isFollowReferrals() {
        return followReferrals;
    }

    void setFollowReferrals(boolean followReferrals) {
        this.followReferrals = followReferrals;
    }

    public String getBaseDn() {
        return baseDn;
    }

    void setBaseDn(String baseDn) {
        this.baseDn = baseDn == null ? null : baseDn.toLowerCase().trim();
    }

    private void setCache(int cacheSize) {
        if (availablePrincipalsCache == null) {
            availablePrincipalsCache = new SimpleCache(new MemoryStash(cacheSize), null, null, new GroupMapImpl());
        } else {
            availablePrincipalsCache.clear();
        }
    }

    RoleMap retrieveRolesFromCache(String filter) {
        RoleMap retrieveFromCache = (RoleMap) retrieveFromCache(ROLES_CACHE_PREFIX, filter);
        if (logger.isDebugEnabled()) {
            if (retrieveFromCache != null && !retrieveFromCache.isEmpty()) {
                logger.debug("Got " + retrieveFromCache.size() + " Roles from the cache");
            } else {
                logger.debug("No Roles in cache");
            }
        }
        return retrieveFromCache;
    }

    Map<String, User> retrieveUsersFromCache(String filter) {
        Map<String, User> retrieveFromCache = (Map<String, User>) retrieveFromCache(USERS_CACHE_PREFIX, filter);
        if (logger.isDebugEnabled()) {
            if (retrieveFromCache != null && !retrieveFromCache.isEmpty()) {
                logger.debug("Got " + retrieveFromCache.size() + " Users from the cache");
            } else {
                logger.debug("No Users in cache");
            }
        }
        return retrieveFromCache;
    }

    private Serializable retrieveFromCache(String prefix, String filter) {
        Serializable retrieved = availablePrincipalsCache.retrieve(getFilter(prefix, filter));
        return retrieved;
    }

    void storeRolesInCache(String filter, RoleMap roles) {
        storeInCache(ROLES_CACHE_PREFIX, filter, roles);
    }

    void storeUsersInCache(String filter, Map<String, User> users) {
        storeInCache(USERS_CACHE_PREFIX, filter, (Serializable) users);
    }

    private void storeInCache(String prefix, String filter, Serializable toCache) {
        if (logger.isDebugEnabled()) {
            logger.debug("Caching " + toCache + " " + prefix);
        }
        availablePrincipalsCache.store(getFilter(prefix, filter), toCache, new Long(getExpiryTime()), null);
    }

    private static String getFilter(String prefix, String filter) {
        return prefix + (filter == null || filter.equals("") ? WILDCARD_SEARCH : filter);
    }

    public void clearCache() {
        availablePrincipalsCache.clear();
    }

    private long getExpiryTime() {
        int ttl = 300000;
        try {
            ttl = timeToLive;
            if (ttl < 30000) {
                logger.warn("Active Directory cache TTL is less than 30 seconds. This would cause serious performance problems. The minimum value of 30 seconds will now be used");
                ttl = 30000;
            }
        } catch (Exception e) {
            // ignore
        }
        return ttl + System.currentTimeMillis();
    }

    void setTimeToLive(int timeToLive) {
        this.timeToLive = timeToLive;
    }

    void setValidOus(String baseDn, Collection<String> includedOuFilterList, Collection<String> excludedOuFilterList) {
        includedOuBasesList.clear();
        includedOuBasesList.addAll(getFormattedOuFilterList(baseDn, includedOuFilterList));
        excludedOuBasesList.clear();
        excludedOuBasesList.addAll(getFormattedOuFilterList(baseDn, excludedOuFilterList));
        includedOuBasesList.removeAll(excludedOuBasesList);

        hasFilteredOus = !includedOuBasesList.isEmpty();
        if (!hasFilteredOus) {
            includedOuBasesList.add(baseDn);
        } 

        if (logger.isDebugEnabled()) {
            logger.debug("Included OU Bases:");
            for (String dn : includedOuBasesList) {
                logger.debug(" " + dn);
            }

            logger.debug("Excluded OU Bases:");
            for (String dn : excludedOuBasesList) {
                logger.debug(" " + dn);
            }
        }
    }

    private static Collection<String> getFormattedOuFilterList(String baseDn, Collection<String> ouFilterList) {
        Collection<String> formattedOuFilterList = new HashSet<String>();
        for (String dn : ouFilterList) {
            if (!dn.trim().toLowerCase().endsWith(baseDn.trim().toLowerCase())) {
                dn = postfixWithComma(dn) + baseDn;
            }
            formattedOuFilterList.add(dn);
        }
        return formattedOuFilterList;
    }

    private void setIncludeStandardUsers(boolean includeStandardUsers) {
        if (includeStandardUsers) {
            if (hasFilteredOus) {
                includedOuBasesList.add(0, appendBaseDn(CN_USERS));
            }
        } else {
            excludedOuBasesList.add(0, appendBaseDn(CN_USERS));
        }
    }

    private void setIncludeBuiltInGroups(boolean includeBuiltInGroups) {
        if (includeBuiltInGroups) {
            if (hasFilteredOus) {
                includedOuBasesList.add(0, appendBaseDn(CN_BUILTIN));
            }
        } else {
            excludedOuBasesList.add(0, appendBaseDn(CN_BUILTIN));
        }
    }

    boolean isUsernamesAreCaseSensitive() {
        return usernamesAreCaseSensitive;
    }

    private void setUsernamesAreCaseSensitive(boolean usernamesAreCaseSensitive) {
        this.usernamesAreCaseSensitive = usernamesAreCaseSensitive;
    }

    boolean isDnValid(String dn) {
        boolean included = isInOuList(includedOuBasesList, dn);
        boolean notExcluded = !isInOuList(excludedOuBasesList, dn);
        return included && notExcluded;
    }

    private static boolean isInOuList(Collection<String> basesList, String dn) {
        for (String dnToCheck : basesList) {
            if (dn.toLowerCase().endsWith(dnToCheck.toLowerCase())) {
                return true;
            }
        }
        return false;
    }

    private void setActiveDirectoryUrls() throws URISyntaxException {
        activeDirectoryUrls.clear();
        lastContactedActiveDirectoryUrl = null;

        URI primaryUri = buildURI(controllerHost, controllerPort);
        activeDirectoryUrls.add(primaryUri);

        for (String uri : getBackupControllerHosts()) {
            if (uri.contains(PORT_SEPARATOR)) {
                activeDirectoryUrls.add(buildURI(uri));
            } else {
                activeDirectoryUrls.add(buildURI(uri, controllerPort));
            }
        }

        setLastContactedActiveDirectoryUrl(primaryUri);
    }

    private static URI buildURI(String host, int port) throws URISyntaxException {
        return buildURI(host + PORT_SEPARATOR + port);
    }

    private static URI buildURI(String url) throws URISyntaxException {
        return new URI(LDAP_PROTOCOL + url);
    }

    String getContactableActiveDirectories() {
        URI lastContactedUrl = getLastContactedActiveDirectoryUrl();
        URI firstUrl = activeDirectoryUrls.isEmpty() ? null : activeDirectoryUrls.iterator().next();
        boolean isDifferent = !lastContactedUrl.equals(firstUrl);

        Collection<URI> hosts = new ArrayList<URI>(activeDirectoryUrls.size() + 1);
        if (isDifferent) {
            hosts.add(lastContactedUrl);
        }
        hosts.addAll(activeDirectoryUrls);
        return getHosts(hosts);
    }

    private static String getHosts(Collection<URI> urls) {
        StringBuffer buffer = new StringBuffer();
        for (Iterator itr = urls.iterator(); itr.hasNext();) {
            URI url = (URI) itr.next();
            buffer.append(url.toString());
            if(itr.hasNext()) {
                buffer.append(" ");
            }
        }
        return buffer.toString();
    }
    
    synchronized URI getLastContactedActiveDirectoryUrl() {
        return lastContactedActiveDirectoryUrl;
    }

    synchronized void setLastContactedActiveDirectoryUrl(String url) {
        try {
            setLastContactedActiveDirectoryUrl(new URI(url));
        } catch (URISyntaxException e) {
            // ignore
        }
    }

    private synchronized void setLastContactedActiveDirectoryUrl(URI url) {
        if (lastContactedActiveDirectoryUrl == null || !lastContactedActiveDirectoryUrl.equals(url)) {
            PropertyList kerbrosControllerSettings = getKerbrosControllerSettings(url);
            propertyManager.refresh(Collections.singletonMap("activeDirectory.backupControllerHosts", kerbrosControllerSettings.getAsPropertyText()));
        }
        lastContactedActiveDirectoryUrl = url;
    }

    private PropertyList getKerbrosControllerSettings(URI contactedUrl) {
        PropertyList values = new PropertyList();
        values.add(getKerbrosController(contactedUrl));

        for (URI url : activeDirectoryUrls) {
            String kerbrosController = getKerbrosController(url);
            if (!values.contains(kerbrosController)) {
                values.add(kerbrosController);
            }
        }
        return values;
    }

    private static String getKerbrosController(URI url) {
        String toParse = url.toString();
        return toParse.substring(LDAP_PROTOCOL.length(), toParse.lastIndexOf(PORT_SEPARATOR));
    }

    void search(InitialLdapContext context, String filter, String[] attributes, PagedResultMapper mapper) throws Exception {
        template.search(context, filter, attributes, mapper);
    }

    void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    private RealmKey getRealmKey(String name) {
        return new RealmKey(name, realm);
    }

    public Object doAs(PrivilegedAction action) throws UserDatabaseException {
        Object result = null;
        if (isServiceAuthenticationGssApi()) {
            try {
                LoginContext context = getServiceAccountLoginContext();
                result = Subject.doAs(context.getSubject(), action);
                logoutContext(context);
            } catch (Exception e) {
                logger.error("Failure to create Login Context", e);
                throw new UserDatabaseException("", e);
            }
        } else {
            result = action.run();
        }
        
        if (result instanceof Throwable) {
            Throwable e = (Throwable) result;
            logger.error("Failure to doAs", e);
            throw new UserDatabaseException("", e);
        }
        return result;
    }
    
    private LoginContext getServiceAccountLoginContext() throws Exception {
        /*
         * Only attempt to load the service account context if it has not been
         * loaded, if the username has changed or if the password has changed
         */
        try {
            return createLoginContext(getServiceAccountName(), getServiceAccountPassword());
        } catch (LoginException e) {
            Throwable cause = e.getCause();
            // Check the class by name to allow non Sun Javas to compile
            if (cause != null && cause.getClass().getName().equals("sun.security.krb5.KrbException")) {
                throw new Exception("Failed to logon. Please check your Active Directory configuration.", e);
            }
            throw e;
        } 
    }

    LoginContext createLoginContext(String username, String password) throws LoginException {
        if (logger.isDebugEnabled()) {
            logger.debug("Creating login context for " + username);
        }

        UserPasswordCallbackHandler callbackHandler = new UserPasswordCallbackHandler();
        callbackHandler.setUserId(username);
        callbackHandler.setPassword(password);

        LoginContext context = new LoginContext(ActiveDirectoryUserDatabase.class.getName(), callbackHandler);
        context.login();
        return context;
    }

    private static void logoutContext(LoginContext context) {
        try {
            context.logout();
        } catch (LoginException e) {
            // ignore
        }
    }

    InitialLdapContext getAuthenticatedContext(String url, Map<String, String> properties) throws NamingException {
        Hashtable<String, String> variables = new Hashtable<String, String>(properties);
        variables.put(Context.SECURITY_AUTHENTICATION, getServiceAuthenticationType());
        if (!isServiceAuthenticationGssApi()) {
            variables.put(Context.SECURITY_PRINCIPAL, getServiceAccountName());
            variables.put(Context.SECURITY_CREDENTIALS, getServiceAccountPassword());
        }
        return getInitialContext(url, variables);
    }

    InitialLdapContext getAuthenticatedContext(String url) throws NamingException {
        return getAuthenticatedContext(url, Collections.<String, String> emptyMap());
    }

    InitialLdapContext getInitialContext(String url, Map<String, String> properties) throws NamingException {
        Hashtable<String, String> variables = new Hashtable<String, String>(properties);
        variables.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        variables.put(Context.PROVIDER_URL, url); // Must use fully qualified hostname

        if (isSslProtcolType()) {
            variables.put("java.naming.ldap.factory.socket", "com.sslexplorer.boot.CustomSSLSocketFactory"); 
            // Add the custom socket factory
        }

        if (isFollowReferrals()) {
            variables.put(Context.REFERRAL, "follow");
        }

        variables.put("com.sun.jndi.ldap.connect.timeout", String.valueOf(getTimeout()));
        variables.put("java.naming.ldap.version", "3");
        variables.put("com.sun.jndi.ldap.connect.pool", "true");
        variables.put("javax.security.sasl.qop", "auth-conf,auth-int,auth");
        variables.put(Context.SECURITY_PROTOCOL, getProtocolType());

        InitialLdapContext context = new InitialLdapContext(variables, null);
        String usedUrl = (String) context.getEnvironment().get(Context.PROVIDER_URL);
        setLastContactedActiveDirectoryUrl(usedUrl);
        return context;
    }

    private int getTimeout() {
        return Property.getPropertyInt(getRealmKey("activeDirectory.connection.timeout")) * 1000;
    }

    private static String splitDomain(String domain) {
        StringBuffer buffer = new StringBuffer();
        for (StringTokenizer tokenizer = new StringTokenizer(domain, "."); tokenizer.hasMoreTokens();) {
            if (buffer.length() > 0) {
                buffer.append(",");
            }
            buffer.append("DC=" + tokenizer.nextToken());
        }
        return buffer.toString();
    }
}