/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.boot;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Default {@link PropertyClass}.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public abstract class AbstractPropertyClass implements PropertyClass {
    
    final static Log log = LogFactory.getLog(AbstractPropertyClass.class);

    // Private instance variables
    private Map<String, PropertyDefinition> definitions;
    private String name;
    private List<PropertyDefinitionCategory> categories;
    private Map<String, PropertyDefinitionCategory> categoryMap;
    private boolean supportsReplacementVariablesInValues;
    private boolean autoCommit = true;
    private Map<AbstractPropertyKey,String> values = new HashMap<AbstractPropertyKey, String>();


	/**
     * Constructor.
     * 
     * @param name property type name
     * @param supportsReplacementVariablesInValues 
     * @throws IllegalArgumentException on illegal type name
     */
    public AbstractPropertyClass(String name, boolean supportsReplacementVariablesInValues) {
        if(name.contains(" ")) {
            throw new IllegalArgumentException("Property type name may not contain spaces.");
        } 
        this.supportsReplacementVariablesInValues = supportsReplacementVariablesInValues;
        definitions = new TreeMap<String, PropertyDefinition>();
        categories = new ArrayList<PropertyDefinitionCategory>();
        categoryMap = new HashMap<String, PropertyDefinitionCategory>();
        this.name = name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#addPropertyDefinitionCategory(int,
     *      com.sslexplorer.properties.PropertyDefinitionCategory)
     */
    public void addPropertyDefinitionCategory(int parentId, PropertyDefinitionCategory category) {
        PropertyDefinitionCategory parent = parentId == -1 ? null : categoryMap.get(String.valueOf(parentId));
        if (parent != null) {
            category.setParent(parent);
            if (parent.contains(category)) {
            	parent.removeCategory(category);
            }
            parent.addCategory(category);
        } else {
        	if(categories.contains(category)) {
                categories.remove(category);
        	}
            categories.add(category);
        }
        categoryMap.put(String.valueOf(category.getId()), category);

    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#getPropertyDefinitionCategory(int)
     */
    public PropertyDefinitionCategory getPropertyDefinitionCategory(int id) {
        return categoryMap.get(String.valueOf(id));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#removePropertyDefinitionCategory(int,
     *      com.sslexplorer.properties.PropertyDefinitionCategory)
     */
    public void removePropertyDefinitionCategory(int parentId, PropertyDefinitionCategory category) {
        PropertyDefinitionCategory parent = parentId == -1 ? null : categoryMap.get(String.valueOf(parentId));
        if (parent != null) {
            parent.removeCategory(category);
        }

    }

    /* (non-Javadoc)
     * @see com.sslexplorer.properties.PropertyType#deregisterPropertyDefinition(java.lang.String)
     */
    public void deregisterPropertyDefinition(String propertyDefinitionName) {
        definitions.remove(propertyDefinitionName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#getName()
     */
    public String getName() {
        return name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#getPropertyDefinition(java.lang.String)
     */
    public PropertyDefinition getDefinition(String name) {
        return definitions.get(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#getPropertyDefinitions()
     */
    public Collection<PropertyDefinition> getDefinitions() {
        List<PropertyDefinition> l = new ArrayList<PropertyDefinition>(definitions.values());
        Collections.sort(l);
        return l;
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#isDefinitionExists(java.lang.String)
     */
    public boolean isDefinitionExists(String name) {
        return definitions.containsKey(name);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#retrieveProperty(com.sslexplorer.boot.AbstractPropertyKey)
     */
    public String retrieveProperty(AbstractPropertyKey key) throws IllegalArgumentException {
        synchronized(values) {
            if(values.containsKey(key)) {
                return (String)values.get(key);
            }
        }
        return retrievePropertyImpl(key);
    }

    /**
     * Retrieve the property from the underlying store.
     *  
     * @param key key 
     * @return value
     * @throws IllegalArgumentException if key is incorrect
     */
    protected abstract String retrievePropertyImpl(AbstractPropertyKey key) throws IllegalArgumentException;

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#storeProperty(com.sslexplorer.boot.AbstractPropertyKey, java.lang.String)
     */
    public synchronized String storeProperty(AbstractPropertyKey key, String value) throws IllegalArgumentException {
        if(!autoCommit) {
            if(values.containsKey(key)) {
                return values.put(key, value);
            }
            else {
                String val = retrieveProperty(key);
                values.put(key, value);
                return val;
            }
        }
        else {
            return storePropertyImpl(key, value);
        }
    }
    
    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#storeProperty(com.sslexplorer.boot.AbstractPropertyKey, java.lang.String)
     */
    protected abstract String storePropertyImpl(AbstractPropertyKey key, String value) throws IllegalArgumentException;

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.properties.PropertyType#registerPropertyDefinition(com.sslexplorer.boot.PropertyDefinition)
     */
    public void registerPropertyDefinition(PropertyDefinition propertyDefinition) {
    	propertyDefinition.init(this);
        definitions.put(propertyDefinition.getName(), propertyDefinition);
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#commit()
     */
    public synchronized void commit() {
        for(AbstractPropertyKey key : values.keySet()) {
            storePropertyImpl(key, values.get(key));
        }
        values.clear();
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#setAutoCommit(boolean)
     */
    public synchronized void setAutoCommit(boolean autoCommit) {
        if(!autoCommit && this.autoCommit && values.size() > 0) {
            throw new RuntimeException("Cannot unset auto commit when there are values to be commited.");
        }
        this.autoCommit = autoCommit;
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#getCategories()
     */
    public Collection<PropertyDefinitionCategory> getCategories() {
        return categories;
    }

    /* (non-Javadoc)
     * @see com.sslexplorer.boot.PropertyClass#isSupportsReplacementVariablesInValues()
     */
    public boolean isSupportsReplacementVariablesInValues() {
        return supportsReplacementVariablesInValues;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(PropertyClass o) {
		return getName().compareTo(o.getName());
	}

    /**
     * Retrieve a property as an integer. The value will be retrieved as a string
     * then converted to a primitive int.
     * <p>
     * If the property has never been set then the default value provided in the
     * {@link PropertyDefinition} will be returned.
     * 
     * @param key property key
     * @return value
     * @throws IllegalArgumentException if property doesn't exist
     */
    public int retrievePropertyInt(AbstractPropertyKey key) throws IllegalArgumentException {
        return Integer.parseInt(retrieveProperty(key));
    }

    /**
     * Retrieve a property as a long. The value will be retrieved as a string then
     * converted to a primitive long.
     * <p>
     * If the property has never been set then the default value provided in the
     * {@link PropertyDefinition} will be returned.
     * 
     * @param key property key
     * @return value
     * @throws IllegalArgumentException if property doesn't exist
     */
    public long retrievePropertyLong(AbstractPropertyKey key) throws IllegalArgumentException {
        return Long.parseLong(retrieveProperty(key));
    }

    /**
     * Retrieve a property as a boolen. The value will be retrieved as a string then
     * converted to a primitive boolean, <code>true</code> if the string value
     * is <i>true</i> otherwise <code>false</code>.
     * <p>
     * If the property has never been set then the default value provided in the
     * {@link PropertyDefinition} will be returned.
     * <p>
     * 
     * @param key property key
     * @return value
     * @throws IllegalArgumentException if property doesn't exist
     */
    public boolean retrievePropertyBoolean(AbstractPropertyKey key) throws IllegalArgumentException {
        return Boolean.parseBoolean(retrieveProperty(key));
    }

    /**
     * Retrieve a property as a list. The value will be retrieved as a string then
     * converted to a {@link PropertyList}.
     * <p>
     * If the property has never been set then the default value provided in the
     * {@link PropertyDefinition} will be returned.
     * 
     * @param key property key
     * @return value
     * @throws IllegalArgumentException if property doesn't exist
     */
    public PropertyList retrievePropertyList(AbstractPropertyKey key) throws IllegalArgumentException {
        return new PropertyList(retrieveProperty(key));
    }
    
}
