package com.sslexplorer.agent;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.multiplex.ChannelFactory;
import com.maverick.multiplex.MultiplexedConnection;
import com.maverick.multiplex.Request;
import com.maverick.multiplex.RequestHandler;
import com.maverick.util.ByteArrayWriter;
import com.sslexplorer.boot.RequestHandlerTunnel;
import com.sslexplorer.boot.VersionInfo;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.profile.ProfilePropertyKey;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.User;

public class AgentTunnel extends MultiplexedConnection implements RequestHandlerTunnel {

	static Log log = LogFactory.getLog(AgentTunnel.class);

	public static final String SYNCHRONIZED_REQUEST = "synchronized@3sp.com";

	MultiplexedConnection con = null;
	String type;
	HashMap properties = new HashMap();
	String id;
	SessionInfo session;
	SyncHandler syncHandler = new SyncHandler();
	int timeoutId;

	/**
	 * Constructor for agent tunnels that are <strong>not</strong> associated
	 * with a UI session.
	 * 
	 * @param id agent ID
	 * @param user
	 * @param type
	 * @param factory
	 */
	public AgentTunnel(String id, User user, String type, ChannelFactory factory) {
		this(id, null, user, type, factory);
	}

	/**
	 * Constructor for agent tunnels that may or may not be associated with a UI
	 * session
	 * 
	 * @param id agent ID
	 * @param session UI session the tunnel is associated with or
	 *        <code>null</code> if if not associated with a user session
	 * @param user
	 * @param type
	 * @param factory
	 */
	public AgentTunnel(String id, SessionInfo session, User user, String type, ChannelFactory factory) {
		super(factory);
		this.id = id;
		this.session = session;
		this.type = type;
		registerRequestHandler(AgentTunnel.SYNCHRONIZED_REQUEST, syncHandler);
	}

	/**
	 * Get the agent ID
	 * 
	 * @return agent ID
	 */
	public String getId() {
		return id;
	}

	public void register() {
		timeoutId = -1;
		if (session.getHttpSession() != null) {
			if (Property.getPropertyBoolean(new ProfilePropertyKey("client.preventSessionTimeoutIfActive", session))) {
				timeoutId = LogonControllerFactory.getInstance().addSessionTimeoutBlock(session.getHttpSession(),
					"SSL-Explorer Agent");
			}
		}

	}

	public void unregister() {
		if (!session.isInvalidating() && timeoutId != -1) {
			LogonControllerFactory.getInstance().removeSessionTimeoutBlock(session.getHttpSession(), timeoutId);
		}
	}

	/**
	 * Get the UI session the agent tunnel is associated with or
	 * <code>null</code> if it is not associated with a tunnel.
	 * 
	 * @return session
	 */
	public SessionInfo getSession() {
		return session;
	}

	public String getType() {
		return type;
	}

	public Object getProperty(String name) {
		return properties.get(name);
	}

	public void setProperty(String name, Object value) {
		properties.put(name, value);
	}

	public void tunnel(InputStream _in, OutputStream _out) {

		if (log.isDebugEnabled())
			log.debug("Starting agent tunnel");

		startProtocol(_in, _out, false);

		if (log.isDebugEnabled())
			log.debug("Stopping agent tunnel");

		try {
			AgentManager.getInstance().removeAgent(this);
		} catch (AgentException e) {
			log.error("Error removing agent", e);
		}

	}

	public boolean waitForSync(long ms) throws InterruptedException {
		synchronized (syncHandler) {
			if (syncHandler.isSyncCompleted()) {
				return true;
			}
			syncHandler.wait(ms);
			return syncHandler.isSyncCompleted();
		}
	}

	public class SyncHandler implements RequestHandler {

		private boolean syncCompleted;

		public boolean isSyncCompleted() {
			return syncCompleted;
		}

		public synchronized boolean processRequest(Request request, MultiplexedConnection connection) {
			try {
				notifyAll();
				ByteArrayWriter baw = new ByteArrayWriter();
				try {
					baw.writeString(VersionInfo.getVersion().toString());
					request.setRequestData(baw.toByteArray());
					return true;
				} catch (IOException e) {
					log.error("Failed to send back server version.");
					return false;
				}
			} finally {
				syncCompleted = true;
			}
		}

		public void postReply(MultiplexedConnection connection) {
			DefaultAgentManager.getInstance().startServices((AgentTunnel) connection);
		}
	}

}
