package com.sslexplorer.agent;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.maverick.multiplex.Request;
import com.maverick.util.ByteArrayReader;
import com.maverick.util.ByteArrayWriter;
import com.sslexplorer.core.CoreException;
import com.sslexplorer.policyframework.LaunchSession;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.util.TicketGenerator;

/**
 * Manages <i>Agent</i> instances.
 * 
 * @author Lee David Painter <a href="mailto: lee@3sp.com">&lt;lee@3sp.com&gt;</a>
 */
public class DefaultAgentManager {

	static Log log = LogFactory.getLog(DefaultAgentManager.class);

	HashMap<String, AgentTunnel> agentsByAgentId = new HashMap<String, AgentTunnel>();
	HashMap<String, AgentTunnel> agentsByLogonTicket = new HashMap<String, AgentTunnel>();

	static DefaultAgentManager instance = null;

	static final String AGENT_ID = "agentId";
	static final String SESSION_INFO = "sessionInfo";
	
	HashMap<Class, AgentService> agentServices = new HashMap<Class, AgentService>();
	ResourceRequestHandler resourceRequestHandler = new ResourceRequestHandler();

	private DefaultAgentManager() {
	}
	
	/**
	 * Get an instance of the agent manager.
	 * 
	 * @return agent manager
	 */
	public static DefaultAgentManager getInstance() {
		return instance == null ? instance = new DefaultAgentManager() : instance;
	}
	
	public List<AgentService> getServices() {
		return new ArrayList(agentServices.values());
	}
	
	public AgentService getService(Class cls) {
		return agentServices.get(cls);
	}
	
	/**
	 * Register an AgentService implementation with the agent.
	 * @param cls
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public void registerService(Class cls) throws InstantiationException, IllegalAccessException {
		agentServices.put(cls, (AgentService)cls.newInstance());
	}
	
	/**
	 * Unregister an AgentService implementation on the agent.
	 * @param cls
	 */
	public void unregisterService(Class cls) {
		agentServices.remove(cls);
	}

	/**
	 * Register a new agent.
	 * 
	 * @param ticket unique ticket
	 * @param session session
	 * @param tunnel agent
	 * @throws AgentException
	 */
	public void registerAgent(String ticket, SessionInfo session, AgentTunnel tunnel) throws AgentException {

		LogonControllerFactory.getInstance().removeAuthorizationTicket(ticket);
		
		if (session != null) {
			
			
			synchronized (session) {
				agentsByAgentId.put(ticket, tunnel);
				agentsByLogonTicket.put(session.getLogonTicket(), tunnel);
				tunnel.setProperty(AGENT_ID, ticket);
				tunnel.setProperty(SESSION_INFO, session);
//				session.notifyAll();

				tunnel.register();
				
				tunnel.addListener(new DefaultAgentStartupListener(session));
				tunnel.registerRequestHandler("getResources@3sp.com", resourceRequestHandler);
				
				// Make sure all services are initialized before agent begins
				if (tunnel != null) {
					for(Iterator it = agentServices.values().iterator();it.hasNext();) {
						((AgentService)it.next()).initializeTunnel(tunnel);
					}
				}
			}
		} else {
			tunnel.close();
			throw new AgentException("");
		}
	}

	

	/**
	 * Inform the agent to open the specified URL with its local default browser
	 * 
	 * @param agent agent
	 * @param url URL to open
	 * @param launchSession launch session
	 * @return return code
	 * @throws CoreException on any error
	 */
	public int openURL(AgentTunnel agent, URL url, LaunchSession launchSession) throws CoreException {

		try {
			ByteArrayWriter msg = new ByteArrayWriter();
			msg.writeString(url.toExternalForm());
			msg.writeString(launchSession.getId());
			Request request = new Request("openURL@3sp.com", msg.toByteArray());
			if (agent.sendRequest(request, true)) {
				if(request.getRequestData()!=null) {
				ByteArrayReader rdr = new ByteArrayReader(request.getRequestData());
				return (int) rdr.readInt();
				}
			}

		} catch (Exception e) {
			throw new CoreException(0, "", e);
		}
		return -1;
	}

	/**
	 * Unregister an agent. If the agent is running it will be shutdown.
	 * 
	 * @param tunnel
	 */
	public void unregisterAgent(AgentTunnel tunnel) {

		if (tunnel != null) {
			
			tunnel.unregister();
			
			String ticket = (String) tunnel.getProperty(AGENT_ID);

			if (log.isDebugEnabled())
				log.debug("Unregistering agent with id " + ticket);

			SessionInfo session = (SessionInfo) tunnel.getProperty(SESSION_INFO);

			
			agentsByAgentId.remove(ticket);
			agentsByLogonTicket.remove(session.getLogonTicket());

			if (tunnel.isRunning()) {
				try {
					tunnel.sendRequest(new Request("shutdown@3sp.com", "".getBytes()), false, 10000);
				} catch (IOException e) {
					log.error("Failed to send shutdown request to agent. The agent may not be responding or the network link may now be down.",
						e);
				}
				tunnel.close();
			}
 
			if(session.getHttpSession() == null) {
				log.info("Non UI session so cleaning up session.");			
				LogonControllerFactory.getInstance().logoff(session.getLogonTicket());
			}
		}
	}

	/**
	 * Unregister an agent given the session it attached to. If the agent is
	 * running it will be shutdown.
	 * 
	 * @param session session
	 */
	public void unregisterAgent(SessionInfo session) {
		unregisterAgent((AgentTunnel) agentsByLogonTicket.get(session.getLogonTicket()));
	}

	/**
	 * Get the agent attached to the specified session
	 * 
	 * @param session session
	 * @return agent
	 */
	public AgentTunnel getAgentBySession(SessionInfo session) {
		if (session == null)
			return null;
		return (AgentTunnel) agentsByLogonTicket.get(session.getLogonTicket());
	}

	/**
	 * Get the session attached to the agent with the provided unique ticket.
	 * 
	 * @param ticket unique ticket of agent
	 * @return session agent is attached to
	 */
	public SessionInfo getSessionByAgentId(String ticket) {
		if (ticket == null)
			return null;

		return (SessionInfo) LogonControllerFactory.getInstance().getAuthorizationTicket(ticket);
	}

	/**
	 * Create a new unique 'pending' ticket for agent authentication and
	 * register it with the logon controller. The returned ticket should be
	 * passed to the agent when it starts up.
	 * 
	 * @param session session
	 * @return pending agent ticket
	 */
	public String registerPendingAgent(SessionInfo session) {

		String ticket = TicketGenerator.getInstance().generateUniqueTicket("AGENT");
		LogonControllerFactory.getInstance().registerAuthorizationTicket(ticket, session);
		return ticket;

	}

	/**
	 * Get if the provided session has an active agent attached to it.
	 * 
	 * @param session session
	 * @return agent attached to session
	 */
	public boolean hasActiveAgent(SessionInfo session) {
		return session != null && agentsByLogonTicket.containsKey(session.getLogonTicket());
	}

	/**
	 * Convenience method to get if the session of the provided request has an
	 * active agent attached to it.
	 * 
	 * @param request request
	 * @return agent attached to session
	 */
	public boolean hasActiveAgent(HttpServletRequest request) {
		SessionInfo session = LogonControllerFactory.getInstance().getSessionInfo(request);
		if (session != null)
			return hasActiveAgent(session);
		else
			return false;
	}

	

	/**
	 * Wait for agent registration and synchronization. When this method exits
	 * the agent should be in a fully active state.
	 * <p>
	 * A timeout value (milliseconds) must be provided. If the process has not
	 * completed by this time the method with exit with <code>false</code>.
	 * Note, its is possible the agent may start after this timeout (e.g. if the
	 * client is running on very slow hardware and / or network or if the
	 * timeout value is too low).
	 * 
	 * @param ticket agents unique ticket
	 * @param timeout time in milliseconds to wait for registration and
	 *        synchronization.
	 * @return boolean indicating whether agent is running or not
	 */
	public boolean waitForRegistrationAndSynchronization(String ticket, int timeout) {
		SessionInfo session = getSessionByAgentId(ticket);
		synchronized (session) {

			// Has the agent already registered?
			if (agentsByAgentId.containsKey(ticket))
				return true;

			long left = timeout;
			try {
				while (left >= 0) {
					if (agentsByAgentId.containsKey(ticket)) {
						AgentTunnel agent = agentsByAgentId.get(ticket);
						if (log.isInfoEnabled()) {
							log.info("Client registered, waiting for client synchronized request");
						}
						if(agent.waitForSync(500)) {
							return true;
						}
//						Request request = agent.waitForRequest("synchronized@3sp.com", left);
//						if (request != null) {
//							if (log.isInfoEnabled()) {
//								log.info("Got client synchronized request");
//							}
//							Request request = performStartup(agent);
//							if(request == null) {
//								return false;
//							}
//							try {
//								ByteArrayWriter baw = new ByteArrayWriter();
//								baw.writeString(VersionInfo.getVersion().toString());
//								request.setRequestData(baw.toByteArray());
//								return agent.sendRequest(request, false);
//							}
//							catch(IOException ioe) {
//								log.error("Failed to reply with server details.", ioe);
//								return false;
//							}
//						} else {
//							log.error("Client failed to synchronized.");
//						}
					}
//					try {
						session.wait(500);
//					} catch (InterruptedException e) {
//					}
					left = left - 500;
				}
			} catch (InterruptedException e) {
			}
		}
		return false;
	}

	public void startServices(AgentTunnel tunnel) {
		if (log.isDebugEnabled()) {
			log.debug("Starting agent services for tunnel");
		}
		for(Iterator it = agentServices.values().iterator();it.hasNext();) {
			((AgentService)it.next()).performStartup(tunnel);
		}			
		if (log.isDebugEnabled()) {
			log.debug("Started agent services for tunnel");
		}
	}



//	protected Request performStartup(AgentTunnel tunnel) {
//		try {
//			if (log.isDebugEnabled()) {
//				log.debug("Performing startup, waiting for client sync request");
//			}
//			Request request = tunnel.waitForRequest("synchronized@3sp.com", 30000);
//			if (log.isDebugEnabled()) {
//				log.debug("Got client synchronization request, continuing with startup");
//			}
//			// Make sure all services are initialized before agent begins
//			if (tunnel != null) {
//				for(Iterator it = agentServices.values().iterator();it.hasNext();) {
//					((AgentService)it.next()).performStartup(tunnel);
//				}
//			}	
//			return request;		
//		} catch (InterruptedException e) {
//		}
//		return null;
//
//	}

	
}
