/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.jdbc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Calendar;

/**
 * Utilities useful when working with JDBC databases.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class JDBCUtil {
    /**
     * Validates if the supplied value is null from the result set. This is
     * useful for primitive type checking.
     * 
     * @param resultSet
     * @param columnName
     * @return true if the column was null.
     * @throws SQLException
     */
    public static boolean isNull(ResultSet resultSet, String columnName) throws SQLException {
        return null == resultSet.getObject(columnName);
    }

    /**
     * Transforms a given column from the java.sql.ResultSet from a
     * java.sql.Timestamp to a java.util.Calendar.
     * 
     * @param resultSet
     * @param columnName
     * @return java.util.Calendar
     * @throws SQLException
     */
    public static Calendar getCalendar(ResultSet resultSet, String columnName) throws SQLException {
        return getCalendar(resultSet.getTimestamp(columnName));
    }

    /**
     * Transforms the supplied java.sql.Timestamp into a java.util.Calendar.
     * 
     * @param timestamp
     * @return java.util.Calendar
     */
    public static Calendar getCalendar(Timestamp timestamp) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(timestamp == null ? System.currentTimeMillis() : timestamp.getTime());
        return calendar;
    }
    
    /**
     * Closes the supplied Connection handling any exceptions that may be thrown.
     * @param connection
     */
    public static void cleanup(Connection connection) {
        if (connection != null) {
            try {
                connection.close();
            } catch (SQLException e) {
                // ignore
            }
        }
    }
    
    /**
     * Closes the supplied Statement handling any exceptions that may be thrown.
     * @param statement
     */
    public static void cleanup(Statement statement) {
        if(statement != null) {
            try {
                statement.close();
            } catch (SQLException e) {
                // ignore
            }
        }
    }
    
    /**
     * Closes the supplied JDBCPreparedStatement handling any exceptions that may be thrown.
     * @param statement
     */
    public static void cleanup(JDBCPreparedStatement statement) {
        if(statement != null) {
            try {
                statement.releasePreparedStatement();
            } catch (SQLException e) {
                // ignore
            }
        }
    }

    /**
     * Closes the supplied ResultSet handling any exceptions that may be thrown.
     * @param resultSet
     */
    public static void cleanup(ResultSet resultSet) {
        if (resultSet != null) {
            try {
                resultSet.close();
            } catch (SQLException e) {
                // ignored
            }
        }
    }
}