/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.policyframework.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.policyframework.NoPermissionException;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.PolicyUtil;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceStack;
import com.sslexplorer.policyframework.ResourceType;
import com.sslexplorer.policyframework.ResourceUtil;
import com.sslexplorer.policyframework.forms.AbstractResourcesForm;
import com.sslexplorer.properties.Property;
import com.sslexplorer.properties.impl.profile.ProfilePropertyKey;
import com.sslexplorer.properties.impl.systemconfig.SystemConfigKey;
import com.sslexplorer.security.Constants;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.table.actions.AbstractPagerAction;

/**
 * Abstract class which adds generic behaviour for resources, extending the
 * {@link com.sslexplorer.table.actions.AbstractPagerAction}
 * 
 * @author James D Robinson <a href="mailto:james@3sp.com">&lt;james@3sp.com&gt;</a>
 * 
 */
public abstract class AbstractResourcesDispatchAction extends AbstractPagerAction {
    protected Permission editPermission;
    protected Permission createPermission;
    protected Permission removePermission;
    protected Permission assignPermission;

    /**
     * Constructor
     */
    public AbstractResourcesDispatchAction() {
        super();
    }

    /**
     * Constructor
     * 
     * @param resourceType
     * @param requiredPermissions
     * @param editPermission
     * @param createPermission
     * @param removePermission
     * @param assignPermission
     */
    public AbstractResourcesDispatchAction(ResourceType resourceType, Permission[] requiredPermissions,
                                           Permission editPermission, Permission createPermission,
                                           Permission removePermission, Permission assignPermission) {
        this(resourceType, requiredPermissions, editPermission, createPermission, removePermission, assignPermission, null);
    }

    /**
     * Constructor for normal resource types that have the standard,
     * Create / Edit / Assign, Edit / Assign, Delete and Assign permissions
     * 
     * @param resourceType resource type
     * @param requiresResources requires actual resources of type
     */
    public AbstractResourcesDispatchAction(ResourceType resourceType, ResourceType requiresResources) {
        this(resourceType, new Permission[] { 
                        PolicyConstants.PERM_EDIT_AND_ASSIGN, PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, 
                        PolicyConstants.PERM_DELETE, PolicyConstants.PERM_ASSIGN  },
                        PolicyConstants.PERM_EDIT_AND_ASSIGN, PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, PolicyConstants.PERM_DELETE,
                        PolicyConstants.PERM_ASSIGN, requiresResources);
    }

    /**
     * Constructor for normal resource types that need their permissions
     * specifically stating.
     * 
     * @param resourceType
     * @param requiredPermissions
     * @param editPermission
     * @param createPermission
     * @param removePermission
     * @param assignPermission
     * @param requiresResources
     */
    public AbstractResourcesDispatchAction(ResourceType resourceType, Permission[] requiredPermissions,
                                           Permission editPermission, Permission createPermission,
                                           Permission removePermission, Permission assignPermission,
                                           ResourceType requiresResources) {
        super(resourceType, requiredPermissions, requiresResources);
        this.editPermission = editPermission;
        this.createPermission = createPermission;
        this.removePermission = removePermission;
        this.assignPermission = assignPermission;
    }

    /* (non-Javadoc)
     * @see org.apache.struts.actions.DispatchAction#unspecified(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        return list(mapping, form, request, response);
    }

    /**
     * Confirm removal of a resource
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward confirmRemove(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                    HttpServletResponse response) throws Exception {
        if (getRemovePermission() != null) {
            if (getResourceType() == null) {
                throw new Exception(
                    "Concrete implementation of AbstractResourcesDispatchAction does not provide the ResourceType that it is maintaining.");
            }
            PolicyUtil.checkPermission(getResourceType(), getRemovePermission(), request);
        }
        return mapping.findForward("confirmRemove");
    }

    /**
     * Create a resource
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward create(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        if (getCreatePermission() != null) {
            if (getResourceType() == null) {
                throw new Exception(
                    "Concrete implementation of AbstractResourcesDispatchAction does not provide the ResourceType that it is maintaining.");
            }
            PolicyUtil.checkPermission(getResourceType(), getCreatePermission(), request);
        }
        return mapping.findForward("create");
    }

    /**
     * Remove a resource.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward remove(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        Resource r= getResourceById(((AbstractResourcesForm)form).getSelectedResource());
        checkValid(r, new Permission[] { getRemovePermission() }, mapping, (AbstractResourcesForm) form, request);
        doRemove(mapping, form, request, response);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(r, getSessionInfo(request));
        return getRedirectWithMessages(mapping, request);
    }
    
    /**
     * Perform the removal of a resource
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @throws Exception
     */
    protected void doRemove(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
    throws Exception {
        getResourceType().removeResource(((AbstractResourcesForm)form).getSelectedResource(), getSessionInfo(request));        
    }

    /**
     * View the resource.
     * 
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward view(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        edit(mapping, form, request, response);
        return mapping.findForward("view");
    }

    /**
     * Show information about the resource
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward forward
     * @throws Exception on any error
     */
    public ActionForward information(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        int id = ((AbstractResourcesForm) form).getSelectedResource();
        Resource r = getResourceType().getResourceById(id);
        try {
            ResourceUtil.checkResourceManagementRights(r, getSessionInfo(request), new Permission[] {  PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN,  PolicyConstants.PERM_EDIT_AND_ASSIGN, PolicyConstants.PERM_DELETE });
        }
        catch(NoPermissionException npe) {
        	ResourceUtil.checkResourceAccessRights(r, getSessionInfo(request));
        }
        request.setAttribute(Constants.REQ_ATTR_INFO_RESOURCE, r);
        return mapping.findForward("resourceInformation");
    }

    /**
     * Edit the resource.
     * 
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward edit(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {

        // Get the selected resource ID and call the abstract getResourceById
        // method to actual the actual resource object
        int id = ((AbstractResourcesForm) form).getSelectedResource();
        Resource r = getResourceById(id);
        if (r == null) {
            throw new Exception("No resource with ID " + id);
        }

        // Make sure this resource is one that is valid
        checkValid(r, new Permission[] { getEditPermission(), getCreatePermission(), getAssignPermission() }, mapping, (AbstractResourcesForm) form, request);
        ResourceStack.pushToEditingStack(request.getSession(), r);
        return mapping.findForward("edit");
    }

    /**
     * Clone the resource.
     * 
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward clone(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        // Get the selected resource ID and call the abstract getResourceById
        // method to actual the actual resource object
        int id = ((AbstractResourcesForm) form).getSelectedResource();
        Resource r = getResourceById(id);
        if (r == null) {
            throw new Exception("No resource with ID " + id);
        }

        // Make sure this resource is one that is valid
        checkValid(r, new Permission[] { getCreatePermission() }, mapping, (AbstractResourcesForm) form, request);
        ResourceStack.pushToEditingStack(request.getSession(), r);
        return mapping.findForward("clone");
    }

    /**
     * Display the resource.
     * 
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return ActionForward
     * @throws Exception
     */
    public ActionForward list(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        return mapping.findForward("display");
    }

    /**
     * Change the selected view to
     * {@link AbstractResourcesForm#ICONS_VIEW}.
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward viewIcons(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        ((AbstractResourcesForm) form).setSelectedView(AbstractResourcesForm.ICONS_VIEW);
        CoreUtil.storeUIState("ui_view_" + ((AbstractResourcesForm)form).getModel().getId() + "_" + getSessionInfo(request).getNavigationContext(), AbstractResourcesForm.ICONS_VIEW, request, response);
        return mapping.findForward("display");
    }

    /**
     * Change the selected view to
     * {@link AbstractResourcesForm#LIST_VIEW}.
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward viewList(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        ((AbstractResourcesForm) form).setSelectedView(AbstractResourcesForm.LIST_VIEW);
        CoreUtil.storeUIState("ui_view_" + ((AbstractResourcesForm)form).getModel().getId() + "_" + getSessionInfo(request).getNavigationContext(), AbstractResourcesForm.LIST_VIEW, request, response);
        return mapping.findForward("display");
    }

    /**
     * @param id
     * @return Resource
     * @throws Exception
     */
    public Resource getResourceById(int id) throws Exception {
        return getResourceType().getResourceById(id);
    }

    /**
     * @param r
     * @param permission
     * @param mapping
     * @param form
     * @param request
     * @throws NoPermissionException
     */
    protected void checkValid(Resource r, Permission[] permission, ActionMapping mapping, AbstractResourcesForm form, HttpServletRequest request)
                    throws NoPermissionException {
        ResourceUtil.checkResourceManagementRights(r, this.getSessionInfo(request), permission);
    }

    /**
     * Get the edit permission.
     * 
     * @return Permission
     */
    public Permission getEditPermission() {
        return editPermission;
    }

    /**
     * Get the assign permission
     * 
     * @return Permission
     */
    public Permission getAssignPermission() {
        return assignPermission;
    }

    /**
     * Get the create permission
     * 
     * @return Permission
     */
    public Permission getCreatePermission() {
        return createPermission;
    }

    /**
     * Get the removal/delete permission
     * 
     * @return Permission
     */
    public Permission getRemovePermission() {
        return removePermission;
    }
    
    protected void saveError(HttpServletRequest request, String message, Resource resource) {
        saveError(request, message, resource.getResourceDisplayName());
    }
    
    protected void saveMessage(HttpServletRequest request, String message, Resource resource) {
        saveMessage(request, message, resource.getResourceDisplayName());
    }
}