/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.properties.attributes;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.util.MessageResources;

import com.sslexplorer.boot.PropertyDefinition;
import com.sslexplorer.boot.TypeMetaListItem;
import com.sslexplorer.boot.Util;
import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.properties.Pair;
import com.sslexplorer.properties.PairListDataSource;

/**
 * Wrapper bean for displaying and editing a attribute and its value.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class AttributeValueItem implements Comparable {
	
	final static Log log = LogFactory.getLog(AttributeValueItem.class);

    private AttributeDefinition definition;
    private String label;
    private Object value;
    private String categoryLabel;
    private String categoryId;
    private int rows, columns;
	private Pair[] listItems;

    /**
     * Constructor.
     * 
     * @param definition definition
     * @param request request
     * @param value initial value
     */
    public AttributeValueItem(AttributeDefinition definition, HttpServletRequest request, String value) {
        super();        

        MessageResources messageResources = null;
        if (definition.getMessageResourcesKey() != null) {
        	messageResources = CoreUtil.getMessageResources(request.getSession(), definition.getMessageResourcesKey());
        }
        
        this.definition = definition;
        this.label = definition.getLabel();

        // Get an internationalised category label if possible
        this.categoryLabel = definition.getCategoryLabel();
        String s = messageResources == null ? null : messageResources.getMessage("attributeCategory." + definition.getCategory()
            + ".title");
        if (s != null && !s.equals("")) {
            this.categoryLabel = s;
        } else {
            if (categoryLabel == null || categoryLabel.equals("")) {
                this.categoryLabel = "Attributes";
            }
        }

        // Get an internationalised label if possible, otherwise use default
        // label, otherwise use name
        s = messageResources == null ? null : messageResources.getMessage("attribute." + definition.getName() + ".title");
        if (s != null && !s.equals("")) {
            label = s;
        } else {
            if (label != null && label.equals("")) {
                label = definition.getName();
            }
        }
        if (getCategoryLabel() != null)
            categoryId = Util.makeConstantKey(getCategoryLabel());

        // 
        this.value = getDefinition().parseValue(value);

        // Process the type meta for any type specific parameters
        if (definition.getType() == PropertyDefinition.TYPE_TEXT_AREA) {
            this.value = value;
            StringTokenizer t = new StringTokenizer(definition.getTypeMeta().equals("") ? "25x5" : definition.getTypeMeta(), "x");
            try {
                columns = Integer.parseInt(t.nextToken());
                rows = Integer.parseInt(t.nextToken());
            } catch (Exception e) {
            }
        } else if (definition.getType() == PropertyDefinition.TYPE_LIST) {
			List<Pair> listItemsList = new ArrayList<Pair>();
            if (!definition.getTypeMeta().startsWith("!")) {
                for (Iterator i = ((List) definition.getTypeMetaObject()).iterator(); i.hasNext();) {
                    TypeMetaListItem item = (TypeMetaListItem) i.next();
                    Pair pair = new Pair(item.getValue(), item.getValue());
                    if (item.getValue().equals(value)) {
                        this.value = pair.getValue();
                    }
                    listItemsList.add(pair);
                }
            }
            else { 
                String className = definition.getTypeMeta().substring(1);
                try {
                    Class clazz = Class.forName(className);
                    Object obj = clazz.newInstance();
                    if(obj instanceof PairListDataSource)
                        listItemsList.addAll(((PairListDataSource)obj).getValues(request));
                    else
                        throw new Exception("Not a PairListDataSource.");
                }
                catch(Exception e) {
                    log.error("Failed to create list data source.", e);                    
                }
                this.value = value;
            }
            listItems = new Pair[listItemsList.size()];
            listItemsList.toArray(listItems);
		} else if (definition.getType() == PropertyDefinition.TYPE_STRING) {
            columns = 25;
            if (!definition.getTypeMeta().equals("")) {
                try {
                    columns = Integer.parseInt(definition.getTypeMeta());
                } catch (NumberFormatException nfe) {
                }
            }
        } else if (definition.getType() == PropertyDefinition.TYPE_INTEGER) {
            columns = 8;
            if (!definition.getTypeMeta().equals("")) {
                try {
                    columns = Integer.parseInt(definition.getTypeMeta());
                } catch (NumberFormatException e) {
                }
            }
        } else if (definition.getType() == PropertyDefinition.TYPE_PASSWORD) {
            columns = 25;
            if (!definition.getTypeMeta().equals("")) {
                try {
                    columns = Integer.parseInt(definition.getTypeMeta());
                } catch (NumberFormatException nfe) {
                }
            }
        }
    }

	/**
	 * Get choices for a type {@link PropertyDefinition#TYPE_LIST}.
	 * 
	 * @return value
	 */
	public Pair[] getListItems() {
		return listItems;
	}

    /**
     * Get the label. This will either the label in the definition itself or a
     * label from a message resource (passed in the constructor) if it exists.
     * 
     * @return label
     */

    public String getLabel() {
        return label;
    }

    /**
     * Get the attribute definition this object wraps.
     * 
     * @return definition
     */
    public AttributeDefinition getDefinition() {
        return definition;
    }

    /**
     * Get the value
     * 
     * @return value
     */
    public Object getValue() {
        return value;
    }

    /**
     * Set the value
     * 
     * @param value value
     */
    public void setValue(Object value) {
        this.value = value;
    }

    /**
     * Get the category label
     * 
     * @return category label
     */
    public String getCategoryLabel() {
        return categoryLabel;
    }

    /**
     * Get the category Id
     * 
     * @return category ID
     */
    public String getCategoryId() {
        return categoryId;
    }

    /**
     * Compare this value item with another based on its definitions sort order
     * and name.
     * 
     * @param arg0 object to compare against.
     * @return comparison
     */
    public int compareTo(Object arg0) {
        int i = (categoryId == null ? "" : categoryId).compareTo(((AttributeValueItem) arg0).categoryId == null ? ""
            : ((AttributeValueItem) arg0).categoryId);
        if (i == 0) {
            i = new Integer(getDefinition().getSortOrder()).compareTo(new Integer(((AttributeValueItem) arg0).getDefinition()
                            .getSortOrder()));
            return i == 0 ? (getDefinition().getName().compareTo(((AttributeValueItem) arg0).getDefinition().getName())) : i;
        } else {
            return i;
        }
    }

    /**
     * Get the number of columns appropriate for this property definition. Only
     * applies to property definitions of type
     * {@link PropertyDefinition#TYPE_PASSWORD},
     * {@link PropertyDefinition#TYPE_STRING}
     * 
     * @return number of columns
     */
    public int getColumns() {
        return columns;
    }

    /**
     * Get the number of row appropriate for this property definition. Only
     * applies to property definitions of type
     * {@link PropertyDefinition#TYPE_TEXT_AREA}.
     * 
     * @return number of rows
     */
    public int getRows() {
        return rows;
    }

    /**
     * For boolean property definitions such as checkkbox that will return
     * <code>true</code> if the value is true.
     * 
     * @return selected.
     */
    public boolean getSelected() {
        return value.equals(Boolean.TRUE);
    }

    /**
     * Set the value as a boolean. For boolean property definitions such as
     * checkbox.
     * 
     * @param selected selected
     */
    public void setSelected(boolean selected) {
        this.value = Boolean.valueOf(selected);
    }
}
