/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.properties.attributes;

import java.util.List;

import org.jdom.Element;
import org.jdom.JDOMException;

import com.sslexplorer.boot.PropertyDefinition;
import com.sslexplorer.boot.XMLPropertyDefinition;

/**
 * Abstract implementation of an {@link AttributeDefinition} that is configured
 * from an XML element.
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 * @see com.sslexplorer.security.UserDatabase
 */
public class XMLAttributeDefinition extends XMLPropertyDefinition implements AttributeDefinition {

    // Private instance variables

    private String label;
    private boolean system;
    private String categoryLabel;
    private String description;
    private boolean replaceable;
    private int visibility;
    private String classSpecificKey;

    /**
     * Construct
     * 
     * @param element XML element
     * @throws JDOMException on parsing error
     */
    public XMLAttributeDefinition(Element element) throws JDOMException {
        super(element);
        replaceable = "true".equalsIgnoreCase(element.getAttributeValue("replaceable"));
        categoryLabel = element.getAttributeValue("categoryLabel");
        description = element.getAttributeValue("description");
        label = element.getAttributeValue("label");
        visibility = USER_USEABLE_ATTRIBUTE;
        if (element.getAttribute("visibility") != null) {
            visibility = element.getAttribute("visibility").getIntValue();
        }

        // Because elements are coming from XML they must be system
        system = true;
    }

    /**
     * Constructor.
     * 
     * @param type type
     * @param name name
     * @param typeMeta type meta
     * @param category category ID or <code>-1</code> to use the categoryLabel
     * @param categoryLabel category label or <code>null</code>
     * @param defaultValue default value
     * @param visibility visibility. See class description
     * @param sortOrder sort order
     * @param messageResourcesKey message resource bundle key
     * @param hidden hidden
     * @param label label
     * @param description description
     * @param system system
     * @param replaceable
     * @param validationString
     */
    public XMLAttributeDefinition(int type, String name, String typeMeta, int category, String categoryLabel, String defaultValue,
                                  int visibility, int sortOrder, String messageResourcesKey, boolean hidden, String label,
                                  String description, boolean system, boolean replaceable, String validationString) {
        super(type, name, typeMeta, category, defaultValue, sortOrder, messageResourcesKey, hidden);
        this.label = label;
        this.system = system;
        this.hidden = hidden;
        this.categoryLabel = categoryLabel;
        this.description = description;
        this.replaceable = replaceable;
        this.validationString = validationString;
        this.visibility = visibility;
        check();
    }

    /**
     * Set the visibility. This should only be called on initial creation.
     * 
     * @param visibility
     * @see #getVisibility()
     */
    public void setVisibility(int visibility) {
        this.visibility = visibility;
    }

    /**
     * Get the visibility. This should only be called on initial creation.
     * 
     * @return visibility
     * @see #setVisibility(int)
     */
    public int getVisibility() {
        return visibility;
    }

    /**
     * Get if this is a <i>System</i> attribute definition. If it is then the
     * def. cannot be edited or removed
     * 
     * @return system definition
     */
    public boolean isSystem() {
        return system;
    }

    /**
     * Set if this is a <i>System</i> attribute definition. If it is then the
     * def. cannot be edited or removed
     * 
     * @param system system definition
     */
    public void setSystem(boolean system) {
        this.system = system;
    }

    /**
     * Get the label for this attribute. If an empty string is supplied the name
     * will be used as the label in the user interface. Note that the actual
     * label displayed to the use may be different if a message resource exists
     * (see class documentation).
     * 
     * @return label
     */
    public String getLabel() {
        return label;
    }

    /**
     * Set the label for this attribute. If an empty string is supplied the name
     * will be used as the label in the user interface. Note that the actual
     * label displayed to the use may be different if a message resource exists
     * (see class documentation).
     * 
     * @param label label
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Get the category label. If this is not <code>null</code> then the
     * category ID will be used to derive the category label from message
     * resources.
     * 
     * @return category label
     */
    public String getCategoryLabel() {
        return categoryLabel;
    }

    /**
     * Set the category label. If this is not <code>null</code> then the
     * category ID will be used to derive the category label from message
     * resources.
     * 
     * @param categoryLabel category label
     */
    public void setCategoryLabel(String categoryLabel) {
        this.categoryLabel = categoryLabel;
    }

    /**
     * Set the label for this attribute. If an empty string is supplied the name
     * will be used as the label in the user interface. Note that the actual
     * description displayed to the use may be different if a message resource
     * exists (see class documentation).
     * 
     * @return description
     */
    public String getDescription() {
        return description;
    }

    /**
     * Set the label for this attribute. If an empty string is supplied the name
     * will be used as the label in the user interface. Note that the actual
     * label displayed to the use may be different if there
     * 
     * @param description description
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Set whether this this attribute may be used for replacements.
     * 
     * @param replaceable replaceable
     */
    public void setReplaceable(boolean replaceable) {
        this.replaceable = replaceable;
    }

    /**
     * Get whether this this attribute may be used for replacements.
     * 
     * @return replaceable
     */
    public boolean isReplaceable() {
        return replaceable;
    }

    /**
     * Set the validation string.
     * 
     * @param validationString validation string
     */
    public void setValidationString(String validationString) {
        this.validationString = validationString;
    }

    /**
     * Get the value of this item as it should be stored as an attribute.
     * 
     * @return formatted attribute value
     */
    public String formatAttributeValue(Object value) {
        if (getType() == PropertyDefinition.TYPE_BOOLEAN) {
            if (getTypeMetaObject() != null && !getTypeMetaObject().equals("")) {
                String trueVal = (String) (((List) getTypeMetaObject()).get(0));
                return value == Boolean.TRUE ? trueVal : (String) (((List) getTypeMetaObject()).get(1));
            }
        }
        return value.toString();
    }

    /**
     * Get the appropriate object for this type.
     * 
     * @param value
     * @return typed value
     */
    public Object parseValue(String value) {
        if (getType() == PropertyDefinition.TYPE_BOOLEAN) {
            if (getTypeMetaObject() != null) {
                String trueVal = (String) (((List) getTypeMetaObject()).get(0));
                return value.equals(trueVal) ? Boolean.TRUE : Boolean.FALSE;
            }
        }
        return value;
    }

}
