/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.security;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sslexplorer.core.CoreServlet;
import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.realms.Realm;

/**
 * Default implementation of a {@link com.sslexplorer.security.UserDatabase}.
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public abstract class DefaultUserDatabase implements UserDatabase {
    static Log log = LogFactory.getLog(DefaultUserDatabase.class);

    // Private instance variables
    protected boolean open;
    protected String description;
    protected boolean supportsAccountCreation;
    protected boolean supportsPasswordChange;
    protected Realm realm;

    /**
     * Constructor.
     * @param description description
     * @param supportsAccountCreation true if concrete user database supports account creation
     * @param supportsPasswordChange true if concrete user database supports password changing
     */
    public DefaultUserDatabase(String description, boolean supportsAccountCreation, boolean supportsPasswordChange) {
        this.description = description;
        this.supportsAccountCreation = supportsAccountCreation;
        this.supportsPasswordChange = supportsPasswordChange;
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#supportsPasswordChange()
     */
    public boolean supportsPasswordChange() {
        return supportsPasswordChange;
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#supportsAccountCreation()
     */
    public boolean supportsAccountCreation() {
        return supportsAccountCreation;
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#getDatabaseDescription()
     */
    public String getDatabaseDescription() {
        return description;
    }

    public void open(CoreServlet controllingServlet, Realm realm) throws Exception {
        if (realm == null) {
            throw new IllegalArgumentException("No realm supplied.");
        }
        this.open = true;
        log.info("Opening user database " + getClass().getName() + " for realm " + realm.getResourceName());
        this.realm = realm;
    }

    public void open(CoreServlet controllingServlet) throws Exception {
        throw new Exception("User databases must be opened with the realm.");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#changePassword(java.lang.String, java.lang.String, java.lang.String, boolean)
     */
    public void changePassword(String username, String oldPassword, String password, boolean forcePasswordChangeAtLogon)
                    throws UserDatabaseException, InvalidLoginCredentialsException {
        assertSupportsPasswordChange();
        throw new InvalidLoginCredentialsException(
                        "User database is not read-only, but the changePassword() method has not been implemented");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#setPassword(java.lang.String, java.lang.String, boolean, com.sslexplorer.security.User, java.lang.String)
     */
    public void setPassword(String username, String password, boolean forcePasswordChangeAtLogon, User adminUser,
                            String adminPassword) throws UserDatabaseException, InvalidLoginCredentialsException {
        assertSupportsPasswordChange();
        throw new InvalidLoginCredentialsException("");

    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#createAccount(java.lang.String, java.lang.String, java.lang.String, java.lang.String com.sslexplorer.security.Role[])
     */
    public User createAccount(String username, String password, String email, String fullname, Role[] roles) throws Exception {
        assertSupportsAccountCreation();
        throw new Exception("User database is not read-only, but the createAccount() method has not been implemented");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#getUsersInRole(com.sslexplorer.security.Role)
     */
    public User[] getUsersInRole(Role role) throws Exception {
        return CoreUtil.getUsersInRole(role, this);
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#updateAccount(com.sslexplorer.security.User, java.lang.String, java.lang.String, com.sslexplorer.security.Role[])
     */
    public void updateAccount(User user, String email, String fullname, Role[] roles) throws Exception {
        assertSupportsAccountCreation();
        throw new Exception("User database is not read-only, but the updateAccount() method has not been implemented");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#deleteAccount(com.sslexplorer.security.User)
     */
    public void deleteAccount(User user) throws Exception, UserNotFoundException {
        assertSupportsAccountCreation();
        throw new Exception("User database is not read-only, but the deleteAccount() method has not been implemented");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#createRole(java.lang.String)
     */
    public Role createRole(String rolename) throws Exception {
        assertSupportsAccountCreation();
        throw new Exception("User database is not read-only, but the createRole() method has not been implemented");
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#deleteRole(java.lang.String)
     */
    public void deleteRole(String rolename) throws Exception {
        assertSupportsAccountCreation();
        throw new Exception("User database is not read-only, but the deleteRole() method has not been implemented");
    }
    
    protected void assertSupportsPasswordChange() throws InvalidLoginCredentialsException {
        if (!supportsPasswordChange()) {
            throw new InvalidLoginCredentialsException("Database doesn't support password change.");
        }
    }
    
    protected void assertSupportsAccountCreation() throws Exception {
        if (!supportsAccountCreation()) {
            throw new Exception("User database is read-only");
        }
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.core.Database#cleanup()
     */
    public void cleanup() throws Exception {
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#getRealm()
     */
    public Realm getRealm() {
        return realm;
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#close()
     */
    public void close() throws Exception {
        this.open = false;
        log.info("Closing user database " + getClass().getName() + " for realm " + realm.getResourceName());
    }

    /*
     * (non-Javadoc)
     * @see com.sslexplorer.security.UserDatabase#isOpen()
     */
    public boolean isOpen() {
        return open;
    }
}