/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.security;

import com.sslexplorer.core.CoreServlet;
import com.sslexplorer.core.Database;
import com.sslexplorer.policyframework.Principal;
import com.sslexplorer.realms.Realm;

/**
 * <p>Implementations of this interface will provide basic user and role related
 * services such as retrieving a user account, retrieving a role, authenticating
 * a user etc.
 * 
 * <p>Some implementations will now support account creation or password
 * changing and as such should return appropriate values for {@link #supportsAccountCreation()}
 * and {@link #supportsPasswordChange()}.
 *  
 *  
 * @author Lee David Painter
 */
public interface UserDatabase extends Database {

    /**
     * Get a readable description of the database.
     * 
     * @return description
     */
    public String getDatabaseDescription();

    /**
     * Get if the database is currently 'open'.
     * 
     * @return open
     */
    public boolean isOpen();

    /**
     * Authenticates the given username/password pair, returning the user object
     * on success or <tt>null</tt> on failure.
     * 
     * @param username
     * @param password
     * @return user object
     * @throws UserDatabaseException
     * @throws InvalidLoginCredentialsException
     * @throws AccountLockedException
     */
    public User logon(String username, String password) throws UserDatabaseException, InvalidLoginCredentialsException,
                    AccountLockedException;

    /**
     * Check the given username/password pair but do not actually logon.
     * <tt>true</tt> is returned on success and <tt>false</tt> on failure.
     * 
     * @param username
     * @param password
     * @return password ok
     * @throws UserDatabaseException
     * @throws InvalidLoginCredentialsException
     */
    public boolean checkPassword(String username, String password) throws UserDatabaseException, InvalidLoginCredentialsException;

    /**
     * Change your password. This method is used by a user to change their own passwords. It
     * is recommended that implementations verify that the old password is correct.
     * 
     * @param username username
     * @param oldPassword 
     * @param password new password
     * @param forcePasswordChangeAtLogon force password change at next logon
     * @throws UserDatabaseException
     * @throws InvalidLoginCredentialsException
     */
    public void changePassword(String username, String oldPassword, String password, boolean forcePasswordChangeAtLogon) throws UserDatabaseException,
                    InvalidLoginCredentialsException;

    /**
     * Set a users password. This is used from the admin pages and requires admin user and password
     * to complete.
     * @param username
     * @param password
     * @param forcePasswordChangeAtLogon
     * @param adminUser
     * @param adminPassword
     * @throws UserDatabaseException
     * @throws InvalidLoginCredentialsException
     */
    public void setPassword(String username, String password, boolean forcePasswordChangeAtLogon, User adminUser, String adminPassword) throws UserDatabaseException, InvalidLoginCredentialsException;
    
    /**
     * Get if this implementation supports changine of passwords
     * 
     * @return password change supported
     */
    public boolean supportsPasswordChange();

    /**
     * Logout a user.
     * 
     * @param user
     */
    public void logout(User user);

    /**
     * List all the users currently registered with the system. This is the list
     * of all users rather than those that are granted access
     * 
     * @param filter filter a filter to apply to the search
     * @return an array of {@link User}s
     * @throws Exception
     */
    public User[] listAllUsers(String filter) throws Exception;

    /**
     * List all the principals currently registered with the system
     * 
     * @return an array of {@link Principal}s
     * @throws Exception
     */
    public Principal[] listAvailablePrincipals() throws Exception;

    /**
     * Get the account details that belong to the given username.
     * 
     * @param username
     * @return user
     * @throws UserNotFoundException if the user could not be found
     * @throws Exception on all other errors
     */
    public User getAccount(String username) throws UserNotFoundException, Exception;

    /**
     * Identify whether this implementation supports the creation of user
     * accounts.
     * 
     * @return <tt>true</tt> if account creation is supported, otherwise
     *         <tt>false</tt>.
     */
    public boolean supportsAccountCreation();

    /**
     * Create a new {@link User}account. This method is optional and should
     * only work when {@link #supportsAccountCreation()} returns <tt>true</tt>.
     * 
     * @param username username
     * @param password password
     * @param email email address
     * @param fullname full name
     * @param roles array of roles
     * @return user user object
     * @throws Exception on any error
     */
    public User createAccount(String username, String password, String email, String fullname, Role[] roles) throws Exception;

    /**
     * Update the details of a {@link User}account. This method is optional and
     * should only work when {@link #supportsAccountCreation()} returns
     * <tt>true</tt>.#
     * 
     * @param user
     * @param email
     * @param fullname
     * @param roles
     * @throws Exception
     */
    public void updateAccount(User user, String email, String fullname, Role[] roles) throws Exception;

    /**
     * Delete a {@link User} account. This method is optional and should only
     * work when {@link #supportsAccountCreation()} returns <tt>true</tt>.
     * 
     * @param user
     * @throws Exception
     * @throws UserNotFoundException 
     */
    public void deleteAccount(User user) throws Exception, UserNotFoundException;


    /**
     * Get a single role given its name
     * 
     * @param rolename role name
     * @return role
     * @throws Exception on any error
     */
    public Role getRole(String rolename) throws Exception;

    /**
     * List all available roles
     * 
     * @param filter filter
     * @return array of roles
     * @throws Exception on any error
     */
    public Role[] listAllRoles(String filter) throws Exception;

    /**
     * Create a new role if the underlying database supports it.
     * 
     * @param rolename role name
     * @return role object
     * @throws Exception on any error
     */
    public Role createRole(String rolename) throws Exception;

    /**
     * Delete a new role
     * 
     * @param rolename role name
     * @throws Exception on any error
     */
    public void deleteRole(String rolename) throws Exception;
    
    /**
     * Get the a list of {@link com.sslexplorer.security.User}s that are in
     * a specified role
     * 
     * @param role role
     * @return users in role
     * @throws Exception on any error
     */
    public User[] getUsersInRole(Role role) throws Exception;
    
    /**
     * @return Realm
     */
    public Realm getRealm();

    /**
     * @param controllingServlet
     * @param realm
     * @throws Exception 
     */
    public void open(CoreServlet controllingServlet, Realm realm) throws Exception;
}