/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.security.actions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.sslexplorer.core.CoreAttributeConstants;
import com.sslexplorer.core.CoreEvent;
import com.sslexplorer.core.CoreEventConstants;
import com.sslexplorer.core.CoreServlet;
import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.core.UserDatabaseManager;
import com.sslexplorer.core.actions.AuthenticatedDispatchAction;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyUtil;
import com.sslexplorer.realms.Realm;
import com.sslexplorer.security.Constants;
import com.sslexplorer.security.GroupsRequiredForUserException;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.Role;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.User;
import com.sslexplorer.security.UserDatabase;
import com.sslexplorer.security.forms.RoleForm;

/**
 * Implementation of an {@link AuthenticatedDispatchAction} that allows an
 * administrator to create or edit a <i>Group</i> (previously known as a
 * <i>Role</i>).
 * 
 * @author Brett Smith <a href="mailto: brett@3sp.com">&lt;brett@3sp.com&gt;</a>
 */
public class ShowRoleDispatchAction extends AuthenticatedDispatchAction {

    /**
     * Constructor.
     */
    public ShowRoleDispatchAction() {
        super(PolicyConstants.ACCOUNTS_AND_GROUPS_RESOURCE_TYPE, new Permission[] { PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN });
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.struts.actions.DispatchAction#unspecified(org.apache.struts.action.ActionMapping,
     *      org.apache.struts.action.ActionForm,
     *      javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request,
                                     HttpServletResponse response) throws Exception {
        return mapping.findForward("display");
    }

    /**
     * Create a new role.
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward create(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
    	UserDatabase udb = UserDatabaseManager.getInstance().getUserDatabase(getSessionInfo(request).getUser().getRealm());
        if (!udb.supportsAccountCreation()) {
            throw new Exception("The underlying user database does not support role creation.");
        }
        PolicyUtil.checkPermission(PolicyConstants.ACCOUNTS_AND_GROUPS_RESOURCE_TYPE, PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, request);
        ((RoleForm) form).initialize(new ArrayList());
        ((RoleForm) form).setReferer(CoreUtil.getReferer(request));
        CoreUtil.addRequiredFieldMessage(this, request);
        return mapping.findForward("display");
    }

    /**
     * Edit an existing role. The role to edit must be placed in the request
     * attribute
     * 
     * @param mapping mapping
     * @param form form
     * @param request request
     * @param response response
     * @return forward
     * @throws Exception on any error
     */
    public ActionForward edit(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        PolicyUtil.checkPermission(PolicyConstants.ACCOUNTS_AND_GROUPS_RESOURCE_TYPE, PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, request);
        Role r = (Role) request.getAttribute(Constants.EDITING_ITEM);
        if (r == null) {
            throw new Exception("No role configured for editing.");
        }
        UserDatabase udb = UserDatabaseManager.getInstance().getUserDatabase(getSessionInfo(request).getUser().getRealm());
        ((RoleForm) form).initialize(Arrays.asList(udb.getUsersInRole(r)));
        ((RoleForm) form).setRolename(r.getPrincipalName());
        ((RoleForm) form).setReferer(CoreUtil.getReferer(request));
        ((RoleForm) form).setEditing();
        CoreUtil.addRequiredFieldMessage(this, request);
        return mapping.findForward("display");
    }

    /**
     * Save the new role or update the existing one depending on whether the
     * role is being edited or created.
     * 
     * @param mapping mappng
     * @param form form
     * @param request request
     * @param response response
     * @return forward forward
     * @throws Exception on any error
     */
    public ActionForward commit(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
                    throws Exception {
        PolicyUtil.checkPermission(PolicyConstants.ACCOUNTS_AND_GROUPS_RESOURCE_TYPE, PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, request);
        RoleForm roleForm = (RoleForm) form;
        UserDatabase udb = UserDatabaseManager.getInstance().getUserDatabase(getSessionInfo(request).getUser().getRealm());
        if (roleForm.getEditing()) {
            try {
                Role r = udb.getRole(
                                roleForm.getRolename());
                List selectedUsers = roleForm.getUserList();
                String[] usersNotRemoved = updateUserRoles(r, selectedUsers, udb.getRealm());
                CoreEvent coreEvent = new CoreEvent(this, CoreEventConstants.GROUP_UPDATED, r, getSessionInfo(request)).addAttribute(
                        CoreAttributeConstants.EVENT_ATTR_PRINCIPAL_ID, r.getPrincipalName());
                
                int j = 0;
                if(!selectedUsers.isEmpty()) {
                    for(Iterator i = selectedUsers.iterator(); i.hasNext(); ) {
                        j++;
                        coreEvent.addAttribute(CoreAttributeConstants.EVENT_ATTR_ACCOUNT + Integer.toString(j),(String)i.next()); 
                    }
                }
                CoreServlet.getServlet().fireCoreEvent(coreEvent);
                saveMessage(request, "availableRoles.roleCreated", roleForm.getRolename());
                
                if (usersNotRemoved.length != 0) {
                    saveError(request, "availableRoles.error.groupsRequired", formatString(usersNotRemoved));
                }
            } catch (Exception ex) {
                CoreServlet.getServlet().fireCoreEvent(
                    new CoreEvent(this, CoreEventConstants.GROUP_UPDATED, null, getSessionInfo(request), ex)
                                    .addAttribute(CoreAttributeConstants.EVENT_ATTR_PRINCIPAL_ID, roleForm.getRolename()));
                throw ex;
            }

        } else {
            try {
                Role r = udb.createRole(roleForm.getRolename());
                List selectedUsers = roleForm.getUserList();
                updateUserRoles(r, selectedUsers, udb.getRealm());

                CoreEvent coreEvent = new CoreEvent(this, CoreEventConstants.GROUP_CREATED, r, getSessionInfo(request)).addAttribute(
                        CoreAttributeConstants.EVENT_ATTR_PRINCIPAL_ID, r.getPrincipalName());
                
                int j = 0;
                if(!selectedUsers.isEmpty()) {
                    for(Iterator i = selectedUsers.iterator(); i.hasNext(); ) {
                        j++;
                        coreEvent.addAttribute(CoreAttributeConstants.EVENT_ATTR_ACCOUNT + Integer.toString(j),(String)i.next()); 
                    }
                }
                CoreServlet.getServlet().fireCoreEvent(coreEvent);
                saveMessage(request, "availableRoles.roleCreated", roleForm.getRolename());
            } catch (Exception ex) {
                CoreServlet.getServlet().fireCoreEvent(
                    new CoreEvent(this, CoreEventConstants.GROUP_CREATED, null, getSessionInfo(request), ex)
                                    .addAttribute(CoreAttributeConstants.EVENT_ATTR_PRINCIPAL_ID, roleForm.getRolename()));
                throw ex;
            }
        }
        // we need to reset the menu items as they could have changed here.
        LogonControllerFactory.getInstance().applyMenuItemChanges(request);
        return cancel(mapping, form, request, response);
    }
    
    private static String formatString(String[] values) {
        if(values.length == 0) {
            return "";
        }
        
        StringBuffer buffer = new StringBuffer();
        for (String value : values) {
            buffer.append(value).append(", ");
        }
        return buffer.substring(0, buffer.length() - 2);
    }
    
    private String[] updateUserRoles(Role r, List selectedUsers, Realm realm) throws Exception {

        /*
         * TODO 
         * 
         * This could be a lot more efficient. The user database currently
         * provides no way of setting roles for users, so we have to jump
         * through hoops by iterating over all users and user updateAccount()
         */
        
    	UserDatabase udb = UserDatabaseManager.getInstance().getUserDatabase(realm);
        User[] u = udb.listAllUsers("*");
        Collection<String> usersNotRemoved = new TreeSet<String>();
        for (int i = 0; i < u.length; i++) {
            Role[] roles = u[i].getRoles();
            
            // Get if the current has the role we are editing
            
            int found = -1;
            for (int j = 0; j < roles.length && found == -1; j++) {
                if (roles[j].getPrincipalName().equals(r.getPrincipalName())) {
                    found = j;
                }
            }
            
            //
            if(found != -1) {
                if (selectedUsers.contains(u[i].getPrincipalName())) {
                    // Selected so leave alone
                } else {
                    // Not selected, remove from role
                    Role[] r2 = new Role[roles.length - 1];
                    System.arraycopy(roles, 0, r2, 0, found);
                    System.arraycopy(roles, found + 1, r2, found, r2.length - found);
                    try {
                        udb.updateAccount(u[i], u[i].getEmail(), u[i].getFullname(), r2);
                    } catch (GroupsRequiredForUserException e) {
                        usersNotRemoved.add(u[i].getPrincipalName());
                    }
                }                
            }
            else {
                // User is not currently in role
                if (selectedUsers.contains(u[i].getPrincipalName())) {
                    // Selected so add
                    Role[] r2 = new Role[roles.length + 1];
                    System.arraycopy(roles, 0, r2, 0, roles.length);
                    r2[roles.length] = r;
                    udb.updateAccount(u[i], u[i].getEmail(), u[i].getFullname(), r2);
                } 
                
            }
        }
        return usersNotRemoved.toArray(new String[usersNotRemoved.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sslexplorer.core.actions.CoreAction#getNavigationContext(org.apache.struts.action.ActionMapping,
     *      org.apache.struts.action.ActionForm,
     *      javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public int getNavigationContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        return SessionInfo.MANAGEMENT_CONSOLE_CONTEXT;
    }
}