/* ========================================================================== *
 * Copyright (C) 2004-2005 Pier Fumagalli <http://www.betaversion.org/~pier/> *
 *                            All rights reserved.                            *
 * ========================================================================== *
 *                                                                            *
 * Licensed under the  Apache License, Version 2.0  (the "License").  You may *
 * not use this file except in compliance with the License.  You may obtain a *
 * copy of the License at <http://www.apache.org/licenses/LICENSE-2.0>.       *
 *                                                                            *
 * Unless  required  by applicable  law or  agreed  to  in writing,  software *
 * distributed under the License is distributed on an  "AS IS" BASIS, WITHOUT *
 * WARRANTIES OR  CONDITIONS OF ANY KIND, either express or implied.  See the *
 * License for the  specific language  governing permissions  and limitations *
 * under the License.                                                         *
 *                                                                            *
 * ========================================================================== */
package com.sslexplorer.vfs.webdav;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;

import com.sslexplorer.core.BundleActionMessage;
import com.sslexplorer.core.CoreException;
import com.sslexplorer.core.CoreUtil;
import com.sslexplorer.policyframework.LaunchSession;
import com.sslexplorer.properties.PropertyProfile;
import com.sslexplorer.security.Constants;
import com.sslexplorer.security.LogonControllerFactory;
import com.sslexplorer.security.SecurityErrorException;
import com.sslexplorer.security.SessionInfo;
import com.sslexplorer.security.User;
import com.sslexplorer.vfs.VFSRepository;
import com.sslexplorer.vfs.VFSResource;

/**
 * <p>
 * A very simple servlet capable of processing very simple <a
 * href="http://www.rfc-editor.org/rfc/rfc2518.txt">WebDAV</a> requests.
 * </p>
 * 
 * <p>
 * Modifications made to integrate with Commons-VFS and SSL-Explorer.
 * </p>
 * 
 * @author <a href="http://www.betaversion.org/~pier/">Pier Fumagalli</a>
 * @author <a href="http://3sp.com/">Brett Smith</a>
 * 
 */
public class DAVServlet implements Servlet, DAVListener {

    final static Log log = LogFactory.getLog(DAVServlet.class);

    /**
     * <p>
     * The session attribute in which the {@link DAVProcessor} is stored
     */
    private static String PROCESSOR_ATTR = "davServlet.processor";

    /**
     * <p>
     * The session attribute in which the {@link SessionInvalidateListener} is
     * stored
     */
    private static String SESSION_INVALIDATE_LISTENER_ATTR = "davServlet.sessionInvalidateListener";

    /**
     * <p>
     * The version of this servlet.
     * </p>
     */
    private static String SIGNATURE = DAVUtilities.getProperty("servlet.signature") + '/' + DAVUtilities.getProperty("version");
    /**
     * <p>
     * The version of this servlet.
     * </p>
     */
    private static String INFORMATION = DAVUtilities.getProperty("servlet.information") + " version "
        + DAVUtilities.getProperty("version");
    /**
     * <p>
     * The {@link ServletContext} associated with this instance.
     * </p>
     */
    private ServletContext context = null;
    /**
     * <p>
     * The {@link ServletConfig} associated with this instance.
     * </p>
     */
    private ServletConfig config = null;
    /**
     * <p>
     * A list of processors in use.
     * </p>
     */
    private static List processors = new ArrayList();

    /**
     * <p>
     * Create a new {@link DAVServlet} instance.
     * </p>
     */
    public DAVServlet() {
        super();
    }

    /**
     * <p>
     * Initialize this {@link Servlet} instance.
     * </p>
     * 
     * <p>
     * The only initialization parameter required by this servlet is the &quot;<code>rootPath</code>&quot;
     * parameter specifying the path of the repository root (either absolute or
     * relative to the configured {@link ServletContext}.
     * </p>
     * 
     * <p>
     * If the specified root is relative, it will be considered to be relative
     * to the {@link ServletContext} deployment path.
     * </p>
     * 
     * <p>
     * In any case, the specified root must ultimately point to an existing
     * directory on a locally-accessible file system.
     * </p>
     * 
     * <p>
     * When set to <code>true</code>, an optional parameter called
     * <code>xmlOnly</code> will force this {@link DAVServlet} to use an
     * {@link XMLRepository} instead of the default {@link VFSRepository}.
     * </p>
     * 
     * <p>
     * Finally, when set to <code>true</code>, the optional parameter
     * <code>debugEnabled</code> will enable logging of method invocation and
     * events in the repository.
     * </p>
     */
    public void init(ServletConfig config) throws ServletException {
        /* Remember the configuration instance */
        this.context = config.getServletContext();
        this.config = config;

    }

    /**
     * <p>
     * Detroy this {@link Servlet} instance.
     * </p>
     */
    public void destroy() {
        for (Iterator i = processors.iterator(); i.hasNext();) {
            ((DAVProcessor) i.next()).getRepository().removeListener(this);
        }
    }

    /**
     * <p>
     * Return the {@link ServletConfig} associated with this instance.
     * </p>
     */
    public ServletConfig getServletConfig() {
        return (this.config);
    }

    /**
     * <p>
     * Return the {@link ServletContext} associated with this instance.
     * </p>
     */
    public ServletContext getServletContext() {
        return (this.context);
    }

    /**
     * <p>
     * Return a informative {@link String} about this servlet.
     * </p>
     */
    public String getServletInfo() {
        return INFORMATION;
    }

    public static DAVSession updateDAVSession(HttpServletRequest request, String resourcePath) {
        // This provides a key on which the store can lookup up webdav session
        // ids

        request.getSession().setAttribute("com.sslexplorer.vfs.webdav.DAVMethod", request.getSession().getId());
        DAVSession webdavSession = DAVSessionStore.getInstance().getSession(request.getSession().getId());
        if (webdavSession == null) {
            webdavSession = new DAVSession(request.getSession().getId());
            DAVSessionStore.getInstance().addSession(webdavSession);
        }
        PropertyProfile profile = (PropertyProfile) request.getSession().getAttribute(Constants.SELECTED_PROFILE);
        if (profile != null) {
            webdavSession.setAttribute(DAVSession.ATTR_PROFILE, profile);
        }
        try {
            User user = LogonControllerFactory.getInstance().getUser(request);
            if (user != null) {
                webdavSession.setAttribute(DAVSession.ATTR_USER, user);
            }
        } catch (SecurityErrorException e1) {
        }
        if (resourcePath != null) {
            webdavSession.setAttribute(DAVSession.ATTR_RESOURCE_PATH, resourcePath);
        }
        return webdavSession;
    }

    public static DAVProcessor getDAVProcessor(HttpServletRequest req) throws CoreException, Exception {
        SessionInfo session = LogonControllerFactory.getInstance().getSessionInfo(req);
        return getDAVProcessor(session);
    }

    public static DAVProcessor getDAVProcessor(SessionInfo session) throws Exception {
        DAVProcessor processor = (DAVProcessor) session.getHttpSession().getAttribute(PROCESSOR_ATTR);
        if (processor == null) {
            VFSRepository repository = VFSRepository.getRepository(session);
            processor = new DAVProcessor(repository, session);
            processors.add(processor);
            session.getHttpSession().setAttribute(PROCESSOR_ATTR, processor);
            session.getHttpSession().setAttribute(SESSION_INVALIDATE_LISTENER_ATTR, new SessionInvalidateListener(processor));
            if (log.isInfoEnabled())
                log.info("Initialized repository");
        }
        return processor;
    }

    public static VFSResource getDAVResource(LaunchSession launchSession, HttpServletRequest request, HttpServletResponse response, String path)
                    throws DAVBundleActionMessageException, Exception {
        VFSResource res = null;
        try {
            DAVProcessor processor = getDAVProcessor(request);
            DAVTransaction transaction = new DAVTransaction(request, response);
            res = processor.getRepository().getResource(launchSession, path, transaction.getCredentials());
            // res.start(transaction);
            res.verifyAccess();
        } catch (DAVAuthenticationRequiredException e) {
            DAVServlet.sendAuthorizationError(request, response, e.getHttpRealm());
            throw e;
        }
        return res;
    }

    /**
     * <p>
     * Execute the current request.
     * </p>
     */
    public void service(ServletRequest request, ServletResponse response) throws ServletException, IOException {
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse res = (HttpServletResponse) response;

        if (System.getProperty("sslexplorer.webdav.debug", "false").equalsIgnoreCase("true")) {
            for (Enumeration e = req.getHeaderNames(); e.hasMoreElements();) {
                String header = (String) e.nextElement();
                for (Enumeration e2 = req.getHeaders(header); e2.hasMoreElements();) {
                    log.info(header + ": " + (String) e2.nextElement());
                }
            }
        }

        if (log.isDebugEnabled())
            log.debug("Processing " + req.getMethod() + " " + req.getPathInfo());

        DAVTransaction transaction = new DAVTransaction(req, res);

        try {

            if (transaction.verifyAuthorization()) {
                DAVServlet.updateDAVSession(req, transaction.getPath());

                DAVProcessor processor = null;
                try {
                    processor = DAVServlet.getDAVProcessor(req);
                } catch (CoreException e) {
                    ActionMessages mesgs = (ActionMessages) request.getAttribute(Globals.ERROR_KEY);
                    if (mesgs == null) {
                        mesgs = new ActionMessages();
                        request.setAttribute(Globals.ERROR_KEY, mesgs);
                    }
                    mesgs.add(Globals.MESSAGE_KEY, e.getBundleActionMessage());
                    return;
                } catch (Exception e1) {
                    throw new IOException(e1.getMessage());
                }

                /**
                 * LDP - JB I have removed the authentication code here and
                 * instead placed in DAVTransaction. This allows the transaction
                 * to correctly obtain sessionInfo through WEBDav.
                 */
                /* Mark our presence */
                res.setHeader("Server", this.context.getServerInfo() + ' ' + SIGNATURE);
                res.setHeader("MS-Author-Via", "DAV");
                res.setHeader("DAV", "1");

                SessionInfo sessionInfo = transaction.getSessionInfo();
                int timeoutId = sessionInfo == null ? -1 : LogonControllerFactory.getInstance()
                                .addSessionTimeoutBlock(transaction.getSessionInfo().getHttpSession(), "DAV Transaction");
                ;
                try {
                    processor.process(transaction);
                } finally {
                    if (timeoutId != -1)
                        LogonControllerFactory.getInstance().removeSessionTimeoutBlock(transaction.getSessionInfo()
                                        .getHttpSession(),
                            timeoutId);
                }
            }
        } catch (DAVRedirection redir) {
            String redirPath = "/fs" + redir.getLocation().getFullPath();
            req.getRequestDispatcher(redirPath).forward(req, res);
        } catch (DAVAuthenticationRequiredException e) {
            // We need to be able to authenticate the SSL-Explorer session was well            
//            sendAuthorizationError(req, res, e.getMount().getMountString());            
            sendAuthorizationError(req, res, e.getHttpRealm());
        } catch(DAVBundleActionMessageException ex) {
            log.error("Network Places Request Failed: " + req.getPathInfo(), ex);
            BundleActionMessage bam = ex.getBundleActionMessage();
            MessageResources mr = CoreUtil.getMessageResources(req.getSession(), bam.getBundle());
            // TODO locale
            String val = mr == null ? null : mr.getMessage(bam.getKey());
            res.sendError(DAVStatus.SC_INTERNAL_SERVER_ERROR, 
                val == null ? ( ex.getMessage() == null ? "No message supplied." : ex.getMessage()) : val);
        } catch (DAVException ex) {
            res.setStatus(ex.getStatus());
        } catch (LockedException ex) {
            res.sendError(DAVStatus.SC_LOCKED, ex.getMessage());
        } catch (Throwable t) {
            log.error("Network Places Request Failed: " + req.getPathInfo(), t);
            res.sendError(DAVStatus.SC_INTERNAL_SERVER_ERROR, t.getMessage() == null ? "<null>" : t.getMessage());
        }
    }

    private boolean isWebDAVMethod(HttpServletRequest request) {
        return

        /*
         * BPS - WE *MUST* support GET at least for get file resources as it is
         * used by application shortcuts HTML templates and other stuff GETing a
         * directory listing iis also very *useful*
         */

        // !request.getMethod().equalsIgnoreCase("get") &&
        !request.getMethod().equalsIgnoreCase("head") && !request.getMethod().equalsIgnoreCase("post");
    }

    public static void sendAuthorizationError(HttpServletRequest request, HttpServletResponse response, String realm)
                    throws IOException {
        if (log.isDebugEnabled())
            log.debug("Sending auth request for realm " + realm);
        response.setHeader("WWW-Authenticate", "Basic realm=\"" + realm + "\"");
        response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
    }

    /**
     * <p>
     * Receive notification of an event occurred in a specific
     * {@link VFSRepository}.
     * </p>
     */
    public void notify(VFSResource resource, int event) {
        String message = "Unknown event";
        switch (event) {
            case DAVListener.COLLECTION_CREATED:
                message = "Collection created";
                break;
            case DAVListener.COLLECTION_REMOVED:
                message = "Collection removed";
                break;
            case DAVListener.RESOURCE_CREATED:
                message = "Resource created";
                break;
            case DAVListener.RESOURCE_REMOVED:
                message = "Resource removed";
                break;
            case DAVListener.RESOURCE_MODIFIED:
                message = "Resource modified";
                break;
        }
        if (log.isDebugEnabled())
            log.debug(message + ": \"" + resource.getRelativePath() + "\"");
    }

    static class SessionInvalidateListener implements HttpSessionBindingListener {

        private DAVProcessor processor;

        /**
         * 
         */
        SessionInvalidateListener(DAVProcessor processor) {
            this.processor = processor;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpSessionBindingListener#valueBound(javax.servlet.http.HttpSessionBindingEvent)
         */
        public void valueBound(HttpSessionBindingEvent arg0) {
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpSessionBindingListener#valueUnbound(javax.servlet.http.HttpSessionBindingEvent)
         */
        public void valueUnbound(HttpSessionBindingEvent arg0) {
            processors.remove(processor);
            // processor.getRepository().removeListener(DAVServlet.this);
        }

    }
}
