/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.testcontainer.policyframework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import javax.servlet.ServletException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.sslexplorer.boot.PropertyList;
import com.sslexplorer.core.CoreServlet;
import com.sslexplorer.navigation.Favorite;
import com.sslexplorer.policyframework.AccessRight;
import com.sslexplorer.policyframework.AccessRights;
import com.sslexplorer.policyframework.DefaultAccessRights;
import com.sslexplorer.policyframework.Permission;
import com.sslexplorer.policyframework.Policy;
import com.sslexplorer.policyframework.PolicyConstants;
import com.sslexplorer.policyframework.PolicyDatabaseFactory;
import com.sslexplorer.policyframework.Resource;
import com.sslexplorer.policyframework.ResourceType;
import com.sslexplorer.realms.Realm;
import com.sslexplorer.security.AuthenticationScheme;
import com.sslexplorer.security.Role;
import com.sslexplorer.security.SystemDatabase;
import com.sslexplorer.security.User;
import com.sslexplorer.testcontainer.AbstractTest;

/**
 * @param <T> 
 */
public abstract class AbstractTestResource<T extends Resource> extends AbstractTest{

    /**
     * @throws Exception
     */
    @Before
    @After
    public void intialize() throws Exception {
        deleteMultipleResources(getAllResources());
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void attachAndDetachResourceFromPolicy() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(resource, selectedPolicies, getSessionInfo());
        assertTrue("The resource should be attached to the policy", getPolicyService().isResourceAttachedToPolicy(resource, policy, realm));
        assertTrue("The resource type must match the resource type of the type created.", getResourceType().equals(resource.getResourceType()));
        
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(resource, getSessionInfo());
        assertFalse("The resource should not be attached to the policy", getPolicyService().isResourceAttachedToPolicy(resource, policy, realm));
        T deletedResource = deleteResource(resource);
        T checkWF = getResource(resource);
        assertNotSame("The resource should have been deleted", deletedResource, checkWF);
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void createAndDeleteAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        assertEquals("There should be one access rights.", getPolicyService().getAccessRights().size(), 1);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        assertEquals("There should be two access rights.", getPolicyService().getAccessRights().size(), 2);
        AccessRights accessRights2 = getPolicyService().getAccessRight(accessRights.getResourceId());
        assertEquals("The acces rights should be the same", accessRights, accessRights2);
        AccessRights accessRights3 = getPolicyService().getAccessRightsByName(accessRights.getResourceName(), realm.getRealmID());
        assertEquals("The acces rights should be the same", accessRights, accessRights3);
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        assertEquals("There should be one access rights.", getPolicyService().getAccessRights().size(), 1);
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void assignAndUnassignAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        
        assertFalse("The resource should not be attached", getPolicyService().isResourceAttachedToPolicy(accessRights, policy, realm));
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        assertTrue("The resource should be attached", getPolicyService().isResourceAttachedToPolicy(accessRights, policy, realm));
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        assertFalse("The resource should not be attached", getPolicyService().isResourceAttachedToPolicy(accessRights, policy, realm));
        
        // other resources should still be attached.
        List<AuthenticationScheme> authenticationSchemeSequences = getSystemService().getAuthenticationSchemeSequences();
        Policy everyone = getPolicyService().getPolicyByName("Everyone", realm.getResourceId());
        for (AuthenticationScheme scheme : authenticationSchemeSequences) {
            assertTrue("Auth scheme " + scheme.getResourceName() + " not assigned to everyone.", getPolicyService().isResourceAttachedToPolicy(scheme, everyone, realm));
        }
        AccessRights personal =  getPolicyService().getAccessRightsByName("Global Permissions", realm.getRealmID());
        assertTrue("Profile " + personal.getResourceName() + " not assigned to everyone.", getPolicyService().isResourceAttachedToPolicy(personal, everyone, realm));
        
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkAllAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        User user = createAccount();
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        assertEquals("The newly created and default.", getPolicyService().getPermittingAccessRights(null, null, null, user).size(), 2);
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, PolicyConstants.PERM_DELETE};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkNoAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        User user = createAccount();
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {};
        assertFalse("The permissions should not be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkDeleteAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        User user = createAccount();
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createDeleteAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkCreateEditAssignAssignAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        User user = createAccount();
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createCreateEditAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkEditAssignAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        int users = getDefaultUserDatabase().listAllUsers("*.*").length ;
        User user = createAccount();
        assertEquals(getDefaultUserDatabase().listAllUsers("*.*").length, users +1);
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createEditAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkAssignAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        User user = createAccount();
        getPolicyService().grantPolicyToPrincipal(policy, user);
        AccessRights accessRights = createAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
        getPolicyService().deleteAccessRights(accessRights.getResourceId());
        getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkResourceAccessRights() throws Exception {
        Realm realm = getDefaultRealm();
    	Policy policy = createPolicy(realm);
    	User user = createAccount();
    	getPolicyService().grantPolicyToPrincipal(policy, user);
    	AccessRights accessRights = createAssignAccessRights(getResourceType().getPermissionClass());
    	PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        T resource = createResource();
        assertFalse("Should not be.", getPolicyService().isPrincipalGrantedResourcesOfType(user, resource.getResourceType(), null));
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(resource, selectedPolicies, getSessionInfo());
        assertTrue("Should be.", getPolicyService().isPrincipalGrantedResourcesOfType(user, resource.getResourceType(), null));
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(resource, getSessionInfo());
        assertFalse("Should not be.", getPolicyService().isPrincipalGrantedResourcesOfType(user, resource.getResourceType(), null));
        
    	getPolicyService().revokePolicyFromPrincipal(policy, user);
    	deleteAccount(user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(accessRights, getSessionInfo());
    	deleteResource(resource);
    	getPolicyService().deleteAccessRights(accessRights.getResourceId());
    	getPolicyService().deletePolicy(policy.getResourceId());
    }

    /**
     * @return T
     * @throws Exception
     */
    public abstract T getNormalResource() throws Exception;
    
    /**
     * @throws Exception
     */
    @Test
    public void createNormalResource() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(resource);
        assertEquals("There should be only one", 1, getAllResources().size());
        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
    }

    /**
     * @return T
     * @throws Exception
     */
    public abstract T getEmptyResource() throws Exception;
    
    /**
     * @throws Exception
     */
    @Test
    public void createEmptyResource() throws Exception {
        T resource = getEmptyResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(resource);
        assertEquals("There should be only one", 1, getAllResources().size());
        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
    }

    /**
     * @return T
     * @throws Exception
     */
    public abstract T getNullResource() throws Exception;
    
    /**
     * @throws Exception
     */
    @Test
    public void createNullResource() throws Exception {
        T resource = getNullResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        
        try {
            createResource(resource);
            fail("This should have failed");
        }
        catch (Exception e) {
            // ignore
        }
        assertTrue("Should have none", getAllResources().isEmpty());
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void createResourceWithSameName() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        
        T createdResourceOne = createResource(resource);
        assertEquals("There should be only one", 1, getAllResources().size());
        T createdResourceTwo = createResource(resource);
        assertEquals("There should be two", 2, getAllResources().size());
        
        deleteResource(createdResourceOne);
        deleteResource(createdResourceTwo);
        assertTrue("Should have none", getAllResources().isEmpty());
    }

    /**
     * @throws Exception
     */
    @Test
    public void createTenResources() throws Exception {
        T normalResource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        List<T> resources = createMultipleResources(normalResource, 10);
        assertEquals("There should be ten resources", 10, getAllResources().size());
        deleteMultipleResources(resources);
        assertTrue("Should have none", getAllResources().isEmpty());
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void updateUnknownResource() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T updatedResource = updateResource(resource);
        assertNull(updatedResource);
    }

    /**
     * @throws Exception
     */
    @Test
    public void updateResourceName() throws Exception {
        T normalResource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(normalResource);
        assertEquals("There should be only one", 1, getAllResources().size());
        
        final String newName = "thisIsMyNewName";
        createdResource.setResourceName(newName);
        T updatedResource = updateResource(createdResource);
        assertEquals("Resource name should have been updated", newName, updatedResource.getResourceName());

        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
    }

    /**
     * @throws Exception
     */
    @Test
    public void updateResourceDescription() throws Exception {
        T normalResource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(normalResource);
        assertEquals("There should be only one", 1, getAllResources().size());
        
        final String newDescription = "thisIsMyNewDescription";
        createdResource.setResourceDescription(newDescription);
        T updatedResource = updateResource(createdResource);
        assertEquals("Resource description should have been updated", newDescription, updatedResource.getResourceDescription());

        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void deleteUnknownResource() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(resource);
        assertEquals("There should be only one", 1, getAllResources().size());
        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void deleteResource() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());

        try {
            deleteResource(resource);
            fail("This should have failed");
        }
        catch(Exception e) {
            // ignore
        }
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void deleteResourceTwice() throws Exception {
        T resource = getNormalResource();
        assertTrue("Should have none", getAllResources().isEmpty());
        T createdResource = createResource(resource);
        assertEquals("There should be only one", 1, getAllResources().size());
        deleteResource(createdResource);
        assertTrue("Should have none", getAllResources().isEmpty());
        
        try {
            deleteResource(createdResource);
            fail("This should have failed");
        }
        catch(Exception e) {
            // ignore
        }
    }
    
    /**
     * @throws Exception
     */
    @Test 
    public void getUnknownResource() throws Exception {
        T normalResource = getNormalResource();
        T foundResource = getResource(normalResource);
        assertNull(foundResource);
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void getResource() throws Exception {
        T normalResource = getNormalResource();
        T createdResource = createResource(normalResource);
        
        T foundResource = getResource(createdResource);
        assertNotNull(foundResource);
        assertEquals("Found same Resource we created", createdResource, foundResource);
        deleteResource(foundResource);
    }
    
    protected static SystemDatabase getSystemService() throws Exception {
        try {
            Class systemDatabaseClass = Class.forName("com.sslexplorer.jdbc.JDBCSystemDatabase");
            SystemDatabase systemDatabase = (SystemDatabase) systemDatabaseClass.newInstance();
            systemDatabase.open(CoreServlet.getServlet());
            return systemDatabase;
        } catch (Exception e) {
            throw new ServletException("Failed to initialise system database.", e);
        }
    }

    protected static Policy createPolicy(Realm realm) throws Exception {
        return createPolicy("Policy A", "Policy A description", Policy.TYPE_NORMAL, realm);
    }
    
    protected static Policy createPolicy(String name, String description, int type, Realm realm) throws Exception {
        return getPolicyService().createPolicy(name, description, type, realm.getRealmID());
    }
    
    protected AccessRights createAllAccessRights(String delegationClass) throws Exception {
        List<AccessRight> accessRights = new ArrayList<AccessRight>();
        accessRights.add(new AccessRight(getResourceType(), PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN));
        accessRights.add(new AccessRight(getResourceType(), PolicyConstants.PERM_DELETE));
        return createAccessRights(delegationClass, accessRights);
    }

    protected AccessRights createCreateEditAssignAccessRights(String delegationClass) throws Exception {
        List<AccessRight> accessRights = Collections.singletonList(new AccessRight(getResourceType(), PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN));
        return createAccessRights(delegationClass, accessRights);
    }
    
    protected AccessRights createEditAssignAccessRights(String delegationClass) throws Exception{
        List<AccessRight> accessRights = Collections.singletonList(new AccessRight(getResourceType(), PolicyConstants.PERM_EDIT_AND_ASSIGN));
        return createAccessRights(delegationClass, accessRights);
    }
    
    protected AccessRights createAssignAccessRights(String delegationClass) throws Exception{
        List<AccessRight> accessRights = Collections.singletonList(new AccessRight(getResourceType(), PolicyConstants.PERM_ASSIGN));
        return createAccessRights(delegationClass, accessRights);
    }
    
    protected AccessRights createDeleteAccessRights(String delegationClass) throws Exception{
        List<AccessRight> accessRights = Collections.singletonList(new AccessRight(getResourceType(), PolicyConstants.PERM_DELETE));
        return createAccessRights(delegationClass, accessRights);
    }
    
    private static AccessRights createAccessRights(String delegationClass, List<AccessRight> accessRights) throws Exception {
        Calendar calendar = Calendar.getInstance();
        DefaultAccessRights defaultAccessRights = new DefaultAccessRights(getDefaultRealm().getRealmID(), 0, "Access1", "First set of access rights.", accessRights, delegationClass, calendar, calendar);
        return getPolicyService().createAccessRights(defaultAccessRights);
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void checkResourceAccessViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        assertFalse("User should not have access", PolicyDatabaseFactory.getInstance().isPrincipalAllowed(user, resource, false));
        getPolicyService().grantPolicyToPrincipal(policy, role);
        assertTrue("User should have access", PolicyDatabaseFactory.getInstance().isPrincipalAllowed(user, resource, false));
        getPolicyService().revokePolicyFromPrincipal(policy, role);
        assertFalse("User should not have access", PolicyDatabaseFactory.getInstance().isPrincipalAllowed(user, resource, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }
    
    /**
     * @throws Exception
     */
    @Test
    public void checkAllAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        assertEquals("The newly created and default.", getPolicyService().getPermittingAccessRights(null, null, null, user).size(), 2);
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN, PolicyConstants.PERM_DELETE};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkNoAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createAllAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {};
        assertFalse("The permissions should not be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkDeleteAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createDeleteAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkCreateEditAssignAssignAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createCreateEditAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkEditAssignAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createEditAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }

    /**
     * @throws Exception
     */
    @Test
    public void checkAssignAccessRightsViaRole() throws Exception {
        User user = createAccount();
        Role role = createRole();
        user = updateAccountRoles(user, Collections.singleton(role));
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        getPolicyService().attachResourceToPolicy(resource, policy, 0, realm);
        
        getPolicyService().grantPolicyToPrincipal(policy, role);
        AccessRights accessRights = createAssignAccessRights(getResourceType().getPermissionClass());
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(accessRights, selectedPolicies, getSessionInfo());
        
        Permission[] permissions = new Permission[] {PolicyConstants.PERM_ASSIGN};
        assertTrue("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_DELETE};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_CREATE_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        permissions = new Permission[] {PolicyConstants.PERM_EDIT_AND_ASSIGN};
        assertFalse("The permissions should be permitted.", getPolicyService().isPermitted(getResourceType(), permissions, user, false));
        
        deleteResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
        user = updateAccountRoles(user, Collections.<Role>emptyList());
        deleteRole(role);
        deleteAccount(user);
    }
    
    protected List<T> createMultipleResources(T resource, int number) throws Exception {
        List<T> resources = new ArrayList<T>(number);
        for(int i = 0; i < number; i++) {
            resource.setResourceName("Resource name " + i);
            resources.add(createResource(resource));
        }
        return resources;
    }
    
    protected void deleteMultipleResources(List<T> resources) throws Exception {
        for (T resource : resources) {
            deleteResource(resource);
        }
    }

    /**
     * @throws Exception
     */
    @Test
    public void testFavorites() throws Exception{
        Realm realm = getDefaultRealm();
        Policy policy = createPolicy(realm);
        T resource = createResource();
        User user = createAccount();
        PropertyList selectedPolicies = PropertyList.createFromArray(new int[] {policy.getResourceId()});
        PolicyDatabaseFactory.getInstance().attachResourceToPolicyList(resource, selectedPolicies, getSessionInfo());
        getPolicyService().grantPolicyToPrincipal(policy, user);
        assertEquals("There should be no favorites", 0, getSystemService().getFavorites(resource.getResourceType().getResourceTypeId(), user).size());
        getSystemService().addFavorite(resource.getResourceType().getResourceTypeId(), resource.getResourceId(), user.getPrincipalName());
        assertEquals("There should now be one favorites", 1, getSystemService().getFavorites(resource.getResourceType().getResourceTypeId(), user).size());
        Favorite f = getSystemService().getFavorite(resource.getResourceType().getResourceTypeId(), user, resource.getResourceId());
        assertEquals("The favorite and resource id should match.", resource.getResourceId(), f.getFavoriteKey());
        assertEquals("The favorite username should match the users principle name.", user.getPrincipalName(), f.getUsername());
        assertEquals("The favorite type id and resource type id should match.", resource.getResourceType().getResourceTypeId(), f.getType());
        getSystemService().removeFavorite(resource.getResourceType().getResourceTypeId(), resource.getResourceId(), user.getPrincipalName());
        assertEquals("There should be no favorites", 0, getSystemService().getFavorites(resource.getResourceType().getResourceTypeId(), user).size());
        getPolicyService().revokePolicyFromPrincipal(policy, user);
        PolicyDatabaseFactory.getInstance().detachResourceFromPolicyList(resource, getSessionInfo());
        deleteResource(resource);
        getResource(resource);
        getPolicyService().deletePolicy(policy.getResourceId());
    }
    
    /**
     * @param resource
     * @return Resource
     * @throws Exception
     */
    public abstract T updateResource(T resource)  throws Exception;
    
    /**
     * @param resource
     * @return Resource
     * @throws Exception
     */
    public abstract T deleteResource(T resource)  throws Exception;
    
    /**
     * @param resource
     * @return Resource
     * @throws Exception
     */
    public abstract T getResource(T resource) throws Exception;
    
    /**
     * @param resource
     * @return Resource
     * @throws Exception
     */
    public abstract T createResource(T resource) throws Exception;
    
    /**
     * @return Resource
     * @throws Exception
     */
    public final T createResource() throws Exception {
        return createResource(getNormalResource());
    }
   
    /**
     * @return Resource
     * @throws Exception
     */
    public abstract ResourceType getResourceType() throws Exception;
    
    /**
     * @return List<Resource>
     * @throws Exception
     */
    public abstract List<T> getAllResources() throws Exception;

    /**
     */
    @ Test
    public abstract void parentMustHaveATest();
}